#!/usr/bin/env python
# -*- coding=utf-8 -*-


import sys
import os
import importlib
from Tools.tips import Tips
from Tools.result import Result
from Tools.formatter import Formatter
from Tools.fileHandle import FileHandle
from Tools.shellCommand import Shell
from Tools.arguments import Arguments
from Tools.cocoapodsTool import CocoapodsTool
from Tools.constant import *
from initSpec import InitSpec


class Init(object):
    __instance = None


    def __init__(self):
        self.formatter = Formatter.instance()
        self.result = Result.instance()
        self.tips = Tips.instance()
        self.fileHandle = FileHandle.instance()
        self.shell = Shell.instance()
        self.arguments = Arguments.instance()
        self.cocoaPodsTool = CocoapodsTool.instance()
        self.initSpec = InitSpec.instance()

    @classmethod
    def instance(cls):
        if cls.__instance is None:
            cls.__instance = Init()
        return cls.__instance

    def create(self):
        """
        创建一个组件库。
        """
        projectPath = self.arguments.projectPath
        projectName = self.arguments.podName
        projectParentDict = self.arguments.projectParentDict
        user = self.arguments.user
        if not self.fileHandle.file_exist(projectParentDict):
            self.formatter.format_warning("父目录" + projectParentDict + "不存在，即将创建此目录")
            self.shell.excommand_until_done(r"sudo mkdir -p " + projectParentDict)
        os.chdir(projectParentDict)
        if self.fileHandle.file_exist(projectPath):
            self.result.returnError("文件已经存在，请确保这个工程是第一次初始化")
        success = self.cocoaPodsTool.libCreate()
        if not success:
            self.result.returnError("模板初始化失败")
        self.formatter.format_print("切换到工程目录")
        # self.shell.excommand_until_done("chmod +x " + projectName)
        os.chdir(projectPath)
        # TODO 去掉耦合
        gerritUrl = "ssh://" + user + "@gerrit.lianjia.com:29418/mobile_ios/" + projectName
        self.formatter.format_print("项目gerrit地址是：" + gerritUrl)
        self.shell.excommand_until_done("git remote add origin " + gerritUrl)
        changeIDHookUrl = user + "@gerrit.lianjia.com:hooks/commit-msg"
        self.formatter.format_print("拉取gerrit的产生changID的脚本")
        self.shell.excommand_until_done(
            "gitdir=$(git rev-parse --git-dir); scp -p -P 29418 " + changeIDHookUrl + " ${gitdir}/hooks/")
        self.formatter.format_print("添加提交脚本review.sh")
        self.fileHandle.writeToFile(reviewInfo, projectPath + "/review.sh")
        if self.fileHandle.file_exist("review.sh"):
            self.formatter.format_print("给提交脚本设置运行权限")
            self.shell.excommand_until_done("chmod +x review.sh")
        self.formatter.format_print("添加文档更新文件VersionHistory.md")
        self.shell.excommand_until_done("touch VersionHistory.md")
        # 更改podSpec文件
        self.initSpec.supportBinary()
        self.addIgnoreFile()
        self.formatter.format_print("初始化git相关")
        self.shell.excommand_until_done("git add .")
        self.shell.excommand_until_done("git commit --amend --no-edit")

        "切换分支因为涉及到需要远端先创建分支，一般没有push权限"
        # formatPrint("创建develop分支并同步到远端")
        # excommandUntilDone("git branch develop")
        # excommandUntilDone("git push origin develop")
        # formatPrint("切换到develop 分支")
        # excommandUntilDone("git checkout develop")
        self.formatter.format_info("组件创建成功：")

    def addIgnoreFile(self):
        self.formatter.format_info("正在处理 git ignore ")
        if not self.fileHandle.file_exist(".gitignore"):
            self.shell.excommand_until_done("touch .gitignore")
        self.fileHandle.writeToFile(ignoreFileString, ".gitignore")


#!/usr/bin/env python
# -*- coding=utf-8 -*-


import sys
import os
import importlib
import yaml
import json
from constant import *
from arguments import Arguments
from formatter import Formatter
from result import Result
from commonFuncs import *
from tips import Tips
from fileHandle import FileHandle
from shellCommand import Shell
from url import URL
from gitTool import GitTool
from podspecTool import PodspecTool


class CocoapodsTool(object):

    __instance = None

    def __init__(self):
        self.formatter = Formatter.instance()
        self.result = Result.instance()
        self.tips = Tips.instance()
        self.fileHandle = FileHandle.instance()
        self.shell = Shell.instance()
        self.arguments = Arguments.instance()
        self.URL = URL.instance()
        self.gitTool = GitTool.instance()
        self.podspecTool = PodspecTool.instance()

    @classmethod
    def instance(cls):
        if cls.__instance is None:
            cls.__instance = CocoapodsTool()
        return cls.__instance

    def libCreate(self):
        """
        初始化spec模板并交互
        """
        self.formatter.format_info("正在初始化工程模板...")
        user = self.arguments.user
        projectName = self.arguments.projectName
        cmd = "pod lib create " + projectName
        pexpect = importlib.import_module('pexpect')
        pexpect.logfile = sys.stdout
        q0 = "What is your name?"
        q1 = "What is your email?"
        q2 = "What platform do you want to use"
        q3 = "What language do you want to use?"
        q4 = "Would you like to include a demo application with your library?"
        q5 = "Which testing frameworks will you use?"
        q6 = "Would you like to do view based testing?"
        q7 = "What is your class prefix?"
        email = user + emailSuffix
        child = pexpect.spawn(cmd)
        isRuning = True
        while isRuning:
            index = child.expect([q0, q1, q2, q3, q4, q5, q6, q7, pexpect.EOF, pexpect.TIMEOUT])
            if index == 0:
                self.formatter.format_print(q0)
                self.formatter.format_print(user)
                child.sendline(user)
            elif index == 1:
                self.formatter.format_print(q1)
                self.formatter.format_print(email)
                child.sendline(email)
            elif index == 2:
                self.formatter.format_print(q2)
                self.formatter.format_print(platform)
                child.sendline(platform)
            elif index == 3:
                self.formatter.format_print(q3)
                self.formatter.format_print(language)
                child.sendline(language)
            elif index == 4:
                self.formatter.format_print(q4)
                self.formatter.format_print(demo)
                child.sendline(demo)
            elif index == 5:
                self.formatter.format_print(q5)
                self.formatter.format_print(testing)
                child.sendline(testing)
            elif index == 6:
                self.formatter.format_print(q6)
                self.formatter.format_print(based)
                child.sendline(based)
            elif index == 7:
                self.formatter.format_print(q7)
                self.formatter.format_print(prefix)
                child.sendline(prefix)
            elif index == 8:
                self.formatter.format_print("初始化模板成功")
                isRuning = False
            elif index == 9:
                isRuning = False
                self.result.returnError("模板初始化超时")
            else:
                self.result.returnError("脚本出错，请检查")
        return not isRuning

    def repoDict(self):
        returnCode, content = self.shell.excommand_until_done("pod repo list")
        if returnCode > 0:
            self.result.returnError("pod repo list 失败")
        # content = matchList("lianjia-mobile_.*", content)[0]
        contentLines = content.split("\n")
        yamlContent = ""
        for line in contentLines:
            if not line or " repo" in line:
                continue
            if ":" not in line:
                line += " :"
            if str(line).startswith("-"):
                line = line.replace("-", " ", 1)
            yamlContent += line + "\n"
        repoDict = yaml.load(yamlContent)
        return repoDict

    def updateRepoList(self, repoList):
        for repoUrl in repoList:
            repoName = self.repoNameFromURL(repoUrl)
            self.updateRepo(repoName)

    def updateRepo(self, repoName):
        self.shell.excommand_until_done("pod repo update " + repoName)

    def addRepo(self, repoURL):
        repoName = self.repoNameFromURL(repoURL)
        userPath = os.path.expanduser('~')
        repoPath = userPath + "/.cocoapods/repos/" + repoName
        if not self.fileHandle.file_exist(repoPath):
            self.shell.excommand_until_done(
                "pod repo add " + repoName + " " + repoURL)

    def cleanRepo(self, repoName):
        userPath = os.path.expanduser('~')
        repoPath = userPath + "/.cocoapods/repos/" + repoName
        if not self.fileHandle.file_exist(repoPath):
            self.result.returnError("要清除的repo不存在")
        else:
            self.gitTool.clean(repoPath)
            self.updateRepo(repoName)

    def pushRepo(self, podName, version, repoUrl):
        # 获取用户目录
        repoPath = self.repoPathByURL(repoUrl)
        if not self.fileHandle.file_exist(repoPath):
            self.addRepo(repoUrl)
        repoName = self.repoNameFromURL(repoUrl)
        self.cleanRepo(repoName)
        self.copyPodspecToRepo(podName, version, repoPath)
        commitMsg = "自动发布" + podName + "的版本：" + version
        self.gitTool.commit(commitMsg, repoPath)
        returnCode, content = self.gitTool.push(repoPath)
        if returnCode > 0:
            self.result.returnError("发布repo失败。请检查发布repo的权限。")

    def push_header_repo(self, pod_name, version, content, repo_url, projectPath):
        repo_name = "lianjia-mobile_ios-" + self.getProjectName(repo_url).lower()
        user_path = os.path.expanduser('~')
        repo_path = user_path + "/.cocoapods/repos/" + repo_name
        if not self.fileHandle.file_exist(repo_path):
            self.shell.excommand_until_done("pod repo add %s %s" % (repo_name, repo_url))
        os.chdir(repo_path)
        self.shell.excommand_until_done("git reset --hard && git clean -fd && git pull")
        pod_file_path = "%s/%s/%s.podspec" % (pod_name, version, pod_name)
        pod_path = os.path.dirname(pod_file_path)
        if not self.fileHandle.file_exist(pod_path):
            self.shell.excommand_until_done("mkdir -p " + pod_path)
        self.fileHandle.writeToFile(content, pod_file_path)
        self.shell.excommand_until_done(
            "git add . && git commit -m \"自动发布组件：%s,版本：%s\" && git push origin" % (pod_name, version))
        os.chdir(projectPath)

    def getProjectName(self, gitUrl=""):
        if not gitUrl or len(gitUrl) == 0:
            return ""
        projectName = gitUrl.replace(".git", "").split("/")[-1]
        return projectName

    def copyPodspecToRepo(self, podName, version, repoPath):
        podspecFileName = podName + ".podspec"
        podspecPath = repoPath + "/" + podName + "/" + version
        if self.fileHandle.file_exist(podspecPath):
            self.formatter.format_error("可能存在相同tag的的podspec，请检查" + podspecPath)
        returnCode, content = self.shell.excommand_until_done("mkdir -p " + podspecPath)
        if returnCode > 0:
            self.result.returnError("创建文件夹失败")
        returnCode, content = self.shell.excommand_until_done(
            "cp ./" + podspecFileName + " " + podspecPath + "/" + podspecFileName)
        if returnCode > 0:
            self.result.returnError("移动临时podspec失败")


    def repoPathByURL(self, repoURL):
        repoName = self.repoNameFromURL(repoURL)
        userPath = os.path.expanduser('~')
        repoPath = userPath + "/.cocoapods/repos/" + repoName
        return repoPath

    def repoNameFromURL(self, repoURL):
        if not self.URL.isURL(repoURL):
            self.result.returnError("url 不合法")
        repoURL = repoURL.strip(".git")
        host, port = self.URL.URLHostAndPort(repoURL)
        hostList = str(host).split(".")
        hostName = ""
        if len(hostList) == 3:
            hostName = hostList[1]
        tmp, path = self.URL.URLPath(repoURL)
        path = str(path).replace("/", "-")
        return hostName + path

    def cleanRepoEnv(self, repoList=[]):
        if len(repoList) == 0:
            repoDict = self.repoDict()
            repoList = repoDict.keys()
        for repoName in repoList:
            self.cleanRepo(repoName)

    def libLint(self):
        podName = self.arguments.podName
        podspecName = podName + ".podspec"
        self.formatter.format_info("正在检测" + podspecName + "是否配置正确（前提是远程已经有写好的Spec文件）。")
        self.podspecTool.backUpPodspec()
        self.podspecTool.changePodSpecForCheck()
        # 太耗时，能用二进制的用二进制
        cmd = "IS_SOURCE=1 pod lib lint " + podspecName + " --allow-warnings --verbose --use-libraries --sources="
        cmd += self.arguments.moduleSources + "  --fail-fast --no-clean"
        returnCode, message = self.shell.excommand_until_done(cmd)
        if returnCode > 0:
            self.result.returnError("podspec 检测不通过，请检查podspec或者提供的dependencyJSON是否有问题。")
        self.podspecTool.restorePodspec()

    def create_tmp_podspec(self, subSpecName):
        commitId = self.arguments.commitId
        podName = self.arguments.podName
        version = self.arguments.version.split("-")[0]
        dependencyDict = self.arguments.dependencyDict
        if not commitId:
            self.result.returnError("subspec 打包必须要commitId")
        self.podspecTool.changePodSpecForTmpPodspec(subSpecName)
        userPath = os.path.expanduser('~')
        # repoSourceList = self.arguments.repoSourceList
        repoSource = self.arguments.moduleSourceList[0]
        repoName = self.repoNameFromURL(repoSource)
        repoPath = userPath + "/.cocoapods/repos/" + repoName
        if not self.fileHandle.file_exist(repoPath):
            returnCode, content = self.shell.excommand_until_done("pod repo add " + repoName + " " + repoSource)
            if returnCode > 0:
                self.result.returnError(repoSource + "添加repo失败，请确认是否再内网。")
        else:
            self.cleanRepo(repoName)
        tmpPodspecFileName = podName + subSpecName + "Tmp.podspec"
        tmpPodspecPath = repoPath + "/" + podName + subSpecName + "Tmp/" + version
        if self.fileHandle.file_exist(tmpPodspecPath):
            self.formatter.format_warning("可能存在已经有的podspec，请检查" + tmpPodspecPath)
        returnCode, content = self.shell.excommand_until_done("mkdir -p " + tmpPodspecPath)
        if returnCode > 0:
            self.result.returnError("创建文件夹失败")
        returnCode, content = self.shell.excommand_until_done(
            "mv ./" + tmpPodspecFileName + " " + tmpPodspecPath + "/" + tmpPodspecFileName)
        if returnCode > 0:
            self.result.returnError("移动临时podspec失败")






#!/usr/bin/env python
# -*- coding=utf-8 -*-

import subprocess
import sys
from formatter import Formatter


class Shell(object):
    __instance = None

    def __init__(self):
        self.formatter = Formatter.instance()

    @classmethod
    def instance(cls):
        if cls.__instance is None:
            cls.__instance = Shell()
        return cls.__instance

    def excommand(cmd):
        """
        子线程执行脚本
        Arguments:
            cmd {str} -- cmd命令
        Returns:
            Pipe -- 管道
        """
        return subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)


    def excommand_until_done(self, cmd):
        """
        子线程执行脚本，直到结束，并输出
        Arguments:
            cmd {str} -- cmd命令
        Returns:
            Pipe -- 管道
        """
        process = subprocess.Popen(cmd, shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE,
                                   stderr=subprocess.STDOUT)
        output = ""
        while process.poll() is None:
            while 1:
                line = process.stdout.readline()
                if line == "":
                    break
                if line:
                    print(line.strip("\n"))
                    output += line
        process.wait()
        if process.returncode > 0:
            sys.stdout.flush()
            sys.stderr.flush()
        return process.returncode, output

    def RunCmd(self, curCmd):
        pipe = subprocess.Popen(['cmd', ""], shell=False, stdout=subprocess.PIPE, \
                                stderr=subprocess.PIPE)
        pipe.stdin.write('%s\n' % curCmd)
        pipe.stdin.close()
        stdoutStr = pipe.stdout.read()
        stderrStr = pipe.stderr.read()
        printStr = ''.join([stdoutStr, stderrStr])
        self.formatter.format_info(printStr)
        return stderrStr, stdoutStr


#!/usr/bin/env python
# -*- coding=utf-8 -*-


import sys
import os
import glob
import json
import shutil
from Tools.tips import Tips
from Tools.result import Result
from Tools.formatter import Formatter
from Tools.fileHandle import FileHandle
from Tools.shellCommand import Shell
from Tools.arguments import Arguments
from Tools.gitTool import GitTool
from Tools.cocoapodsTool import CocoapodsTool
from Tools.podspecTool import PodspecTool
from package import Package
from check import Check
from Tools.resultModel import ResultModel


class AutoPackage(object):
    __instance = None

    def __init__(self):
        self.formatter = Formatter.instance()
        self.result = Result.instance()
        self.tips = Tips.instance()
        self.fileHandle = FileHandle.instance()
        self.shell = Shell.instance()
        self.arguments = Arguments.instance()
        self.gitTool = GitTool.instance()
        self.cocoapodsTool = CocoapodsTool.instance()
        self.podspecTool = PodspecTool.instance()
        self.package = Package.instance()
        self.resultModel = ResultModel.instance()

    @classmethod
    def instance(cls):
        if cls.__instance is None:
            cls.__instance = AutoPackage()
        return cls.__instance

    def cpCodeAndSourcesToCAFPath(self):
        CAFPath = self.arguments.CAFPath
        projectPath = self.arguments.projectPath
        podName = self.arguments.podName
        if not self.fileHandle.file_exist(CAFPath):
            self.shell.excommand_until_done("mkdir -p " + CAFPath)
        # 找到源码和资源位置。因为有些库不规范，只能从podspec下手
        specDict = self.podspecTool.getSpecDict(podName)
        if "preserve_paths" not in specDict.keys():
            self.result.returnError("podspec 不符合规范，必须要有preserve_paths 字段")
        preserve_paths = specDict["preserve_paths"]
        if isinstance(preserve_paths, str):
            preserve_paths = [preserve_paths]
        for path in preserve_paths:
            if str(path).startswith(podName + "/Framework/") or str(path).startswith(podName + "/Archive/"):
                continue
            fileList = self.myGlob(path)
            for filePath in fileList:
                relativePath = str(filePath).replace(projectPath, "").rstrip("/")
                # 拼接目标文件或者文件加的绝对路径
                abstargetPath = os.path.join(CAFPath, relativePath)
                # 是文件就进行复制
                if os.path.isdir(filePath):
                    if not self.fileHandle.file_exist(abstargetPath):
                        self.shell.excommand_until_done("mkdir -p " + abstargetPath)
                elif os.path.isfile(filePath):
                    targetDic = os.path.dirname(abstargetPath)
                    if not self.fileHandle.file_exist(targetDic):
                        self.shell.excommand_until_done("mkdir -p \"" + targetDic + "\"")
                    self.shell.excommand_until_done("cp -R -p \"" + filePath + "\" \"" + abstargetPath + "\"")

    def autoPackage(self):
        projectPath = self.arguments.projectPath
        os.chdir(projectPath)
        podName = self.arguments.podName
        version = self.arguments.version
        commitId = self.arguments.commitId
        branch = self.arguments.branch
        self.resultModel.recordResultDict()
        self.resultModel.recordVersionHistory()
        self.formatter.format_info("正在清理现场")
        self.gitTool.clean()
        self.formatter.format_info("正在拉取远程代码")
        self.gitTool.pull()
        # 判断branch和commit
        self.formatter.format_info("正在切换到分支" + branch)
        self.gitTool.checkout(branch)
        self.arguments.branch, self.arguments.commitId = self.gitTool.currentBranchAndCommitID()
        # 清理过期的库
        self.cleanDeprecatedStaticLibraries()

        if self.arguments.check:
            self.formatter.format_info("执行打包前检测")
            Check.instance().libLint()

        self.formatter.format_info("正在处理静态库目标文件")
        CAFPath = self.arguments.CAFPath
        self.shell.excommand_until_done("rm -rf " + CAFPath)
        if not self.fileHandle.file_exist(CAFPath):
            self.shell.excommand_until_done("mkdir -p " + CAFPath)
        if CAFPath:
            self.cpCodeAndSourcesToCAFPath()
        # 开始打静态库。
        self.package.package()

        if CAFPath:
            self.cpCodeAndSourcesToCAFPath()
        self.resultModel.saveVersionHistory()
        if self.arguments.autoPush:
            self.formatter.format_info("正在自动提交")
            commitMessage = "自动打静态库:" + podName + " " + version
            returnCode, content = self.shell.excommand_until_done("git status")
            if "nothing to commit, working tree clean" not in content:
                self.gitTool.commit(commitMessage)
            # returnCode, newCommitId = self.shell.excommand_until_done("git rev-parse " + branch)
            self.podspecTool.changePodspecForPush()
            returnCode, content = self.shell.excommand_until_done("git status")
            if "nothing to commit, working tree clean" not in content:
                self.shell.excommand_until_done("git add . ")
                self.shell.excommand_until_done("git commit -m \'发布" + podName + "-" + version + "\'")
            returnCode, content = self.shell.excommand_until_done("git push origin")
            if returnCode > 0:
                self.result.returnError("git push 失败，请确认你是否有权限")
            ResultModel.instance().resultDict["autoPushSuccess"] = True
        if self.arguments.autoPushRepo:
            if not self.arguments.repoSources:
                self.result.returnError("自动发布请指定发布源参数：--repoSources")
            self.formatter.format_info("正在自动发布")
            tagVersion = podName + "-" + version
            self.shell.excommand_until_done("git tag " + tagVersion)
            returnCode, content = self.shell.excommand_until_done("git push origin " + tagVersion)
            if returnCode > 0:
                self.result.returnError("tag push 失败"+ content)

            # 网最新的repo里添加数据
            header_repo_list = []
            if "LJPlatBPodSpecs" in self.arguments.repoSources:
                header_repo_list.append("http://gerrit.lianjia.com/mobile_ios/LJPlatBHeaderPodSpecs")
            if "LJPlatCPodSpecs" in self.arguments.repoSources:
                header_repo_list.append("http://gerrit.lianjia.com/mobile_ios/LJPlatCHeaderPodSpecs")
            spec_dict = self.podspecTool.getSpecDict(podName)
            url = ""
            if "source" in spec_dict and "git" in spec_dict["source"]:
                url = spec_dict["source"]["git"]
            header_content = self.podspecTool.podspecForPushHeaderRepo(podName, version, url)
            for repo_url in header_repo_list:
                self.cocoapodsTool.push_header_repo(podName, version, header_content, repo_url, projectPath)
            self.podspecTool.changePodspecForPushRepo()
            # 最好先清空下repo，防止repo污染
            repoSourceList = self.arguments.repoSourceList
            for repoSource in repoSourceList:
                self.formatter.format_info("正在发布到：" + repoSource)
                self.cocoapodsTool.pushRepo(podName, version, repoSource)
            ResultModel.instance().resultDict["autoPushRepoSuccess"] = True
        # 保存结果信息
        # if self.arguments.releasePackage or self.arguments.debugPackage:
        self.resultModel.saveResultPath()
        if self.arguments.releasePackage:
            self.resultModel.uploadPodInfo()
        self.formatter.format_info("二进制打包成功")

    def canPackage(self, subspecName=""):
        podName = self.arguments.podName
        specDict = self.getSpecDict(podName)
        sourcePathList = []
        if not subspecName:
            if "source_files" in specDict.keys():
                sourcePath = specDict["source_files"]
                if isinstance(sourcePath, str):
                    sourcePath = [sourcePath]
                sourcePathList.extend(sourcePath)
            if "subspecs" in specDict.keys():
                subspecs = specDict["subspecs"]
                for subspec in subspecs:
                    if "source_files" in subspec.keys():
                        sourcePath = subspec["source_files"]
                        if isinstance(sourcePath, str):
                            sourcePath = [sourcePath]
                        sourcePathList.extend(sourcePath)
        else:
            if "subspecs" in specDict.keys():
                subspecs = specDict["subspecs"]
                for subspec in subspecs:
                    if "name" in subspec.keys():
                        if subspecName != subspec["name"]:
                            continue
                    if "source_files" in subspec.keys():
                        sourcePath = subspec["source_files"]
                        if isinstance(sourcePath, str):
                            sourcePath = [sourcePath]
                        sourcePathList.extend(sourcePath)

        for sourcePath in sourcePathList:
            splitString = ""
            if "/**/*" in sourcePath:
                splitString = "/**/*"
            elif "/**/" in sourcePath:
                splitString = "/**/"
            elif "/*" in sourcePath:
                splitString = "/*"
            else:
                splitString = "/"
            tmpList = sourcePath.split(splitString)
            intersection = ["m", "mm", "c", "cpp", "cc"]
            if len(tmpList) >= 2:
                if tmpList[-1] != "":
                    suffixList = tmpList[-1].strip("{").strip("}").replace(".", "").split(",")
                    usefulSuffixList = ["m", "mm", "c", "cpp", "cc"]
                    intersection = [val for val in suffixList if val in usefulSuffixList]
                    if not intersection:
                        continue
                    sourcePath = sourcePath.rstrip(tmpList[-1])
            sourcePath = sourcePath.rstrip(splitString)
            for suffix in intersection:
                files = self.findfiles(sourcePath, "." + suffix, True)
                if not files:
                    continue
                return True
        return False

    def findfiles(self, dirname, suffix, recursion=False):
        result = []
        for fileName in os.listdir(dirname):
            filePath = os.path.join(dirname, fileName)
            if os.path.isfile(filePath):
                if filePath.endswith(suffix):
                    result.append(filePath)
            elif recursion and os.path.isdir(filePath):
                result.extend(self.findfiles(filePath, suffix, recursion))
        return result

    def cleanDeprecatedStaticLibraries(self):
        self.formatter.format_info("正在清理陈旧的库")
        frameWorkPath = os.path.join(self.arguments.podName, "Framework")
        if self.arguments.framework and (self.arguments.debugPackage or self.arguments.releasePackage):
            if self.arguments.subPackage:
                self.shell.excommand_until_done("rm -r -f " + frameWorkPath)
            else:
                self.shell.excommand_until_done("rm -r -f " + frameWorkPath)


    def myGlob(self, path, recursion=False):
        # 绝对路径查找
        # TODO 有问题，当写上后缀的时候。
        dirname = os.path.dirname(path)
        basename = os.path.basename(path)
        if "/**/" in path:
            recursion = True
            dirname = path.split("/**/")[0]
            basename = path.split("/**/")[1].replace(".{h,m,mm,c,cpp,cc}", "")
        if not self.fileHandle.file_exist(dirname):
            return []
        fileList = glob.glob(os.path.join(dirname, basename))
        if recursion:
            for fileName in os.listdir(dirname):
                fileDir = os.path.join(dirname, fileName)
                nestBasename = os.path.join(fileDir, basename)
                if os.path.isdir(fileDir):
                    nextFileList = self.myGlob(nestBasename, recursion)
                    if nextFileList:
                        fileList.extend(nextFileList)
        return list(set(fileList))



#!/usr/bin/env python
# -*- coding=utf-8 -*-

from formatter import Formatter
from arguments import Arguments
from mdFormatter import MDFormatter
from shellCommand import Shell
from fileHandle import FileHandle
import json
import os
import requests


class ResultModel(object):
    __instance = None

    def __init__(self):
        self.formatter = Formatter.instance()
        self.mdFormatter = MDFormatter.instance()
        self.arguments = Arguments.instance()
        self.resultDict = {}
        self.versionHistory = ""


    @classmethod
    def instance(cls):
        if cls.__instance is None:
            cls.__instance = ResultModel()
        return cls.__instance

    def recordResultDict(self):
        arguments = Arguments.instance()
        self.resultDict = arguments.argumentDict()

    def recordVersionHistory(self):
        self.versionHistory += "\n"
        self.versionHistory += self.mdFormatter.title(self.arguments.podName + " " + self.arguments.version, 2)
        disorderList = ["podName: " + self.arguments.podName, "version: " + self.arguments.version]
        if self.arguments.branch:
            disorderList.append("branch: " + self.arguments.branch)
        if self.arguments.commitId:
            disorderList.append("version: " + str(self.arguments.version))
        disorderList.append("subPackage: " + str(self.arguments.subPackage))
        disorderList.append("打的静态库包有:")

        if self.resultDict.has_key("dependency"):
            disorderList.append("dependency", str(self.resultDict["dependency"]))
        self.versionHistory += self.mdFormatter.disorderListString(disorderList)
        headerList = [" \\ ", " DebugPackage ", " ReleasePackage "]
        tableList = [" Framework "]
        if self.arguments.framework:
            if self.arguments.debugPackage:
                tableList.append(" ✔️ ")
            else:
                tableList.append("  ")
            if self.arguments.releasePackage:
                tableList.append(" ✔️ ")
            else:
                tableList.append("  ")
        else:
            tableList.append("  ")
            tableList.append("  ")
        self.versionHistory += self.mdFormatter.table([tableList], headerList)

    def saveResultPath(self):
        resultPath = self.arguments.resultPath
        if resultPath:
            if not FileHandle.instance().file_exist(resultPath):
                parentDir = os.path.dirname(resultPath)
                self.formatter.format_info("文件不存在，正在创建输出文件")
                Shell.instance().excommand_until_done("mkdir -p " + parentDir)
                Shell.instance().excommand_until_done("touch " + resultPath)
            FileHandle.instance().writeToFile(json.dumps(self.resultDict), resultPath)
            self.formatter.format_info("结果JSON写入成功"+ resultPath)
        self.formatter.format_info(str(self.resultDict))

    def uploadPodInfo(self):
        # 上传给持续集成平台。
        # http://wiki.lianjia.com/pages/viewpage.action?pageId=412029828
        # Debug
        url = "http://test-app.api.ones.ke.com/plugins/registerComponentByBinary"
        headers = {"Content-Type": "application/json"}
        response = requests.post(url, data=json.dumps(self.resultDict), headers=headers)
        self.formatter.format_info(str(json.loads(response.text)))
        # online
        url = "http://app.api.ones.ke.com/plugins/registerComponentByBinary"
        headers = {"Content-Type": "application/json"}
        response = requests.post(url, data=json.dumps(self.resultDict), headers=headers)
        self.formatter.format_info(str(json.loads(response.text)))


    def saveVersionHistory(self):
        self.formatter.format_info("将打包信息写入VersionHistory文件")
        self.formatter.format_info(self.versionHistory)
        versionHistoryPath = os.path.join(self.arguments.projectPath, "VersionHistory.md")
        FileHandle.instance().appendToFile(self.versionHistory, versionHistoryPath)




#!/usr/bin/env python
# -*- coding=utf-8 -*-


import sys
import os
realpath=os.path.realpath(__file__)
dirPath = os.path.dirname(realpath)
sys.path.append(dirPath)
from Tools.tips import Tips
from Tools.result import Result
from Tools.formatter import Formatter
from Tools.arguments import Arguments
from Features.env import Env
from Features.init import Init
from Features.initSpec import InitSpec
from Features.package import Package
from Features.autoPackage import AutoPackage
from Features.check import Check

__author__ = "handa"
__version__ = "1.0.36"


def checkopts():
    """检测输入参数

    Arguments:
        user {str} -- 用户名
        projectPath {str} -- 项目目录
    """
    if len(sys.argv) == 1:
        Tips.instance().showTips()
        Result.instance().returnError()
    # 先检查环境 TODO open
    # Env.instance().runtimeEnv()
    # 接着看参数
    arg = sys.argv[1]
    if arg == "version":
        Formatter.instance().format_info(__version__)
    elif arg == "initSpec":
        InitSpec.instance().supportBinary()
    elif arg == "init":
        Init.instance().create()
    elif arg == "check":
        Check.instance().check()
    elif arg == "package":
        Package.instance().package()
    elif arg == "autoPackage":
        AutoPackage.instance().autoPackage()
    else:
        Tips.instance().showTips()

def main():
    # 调用接口
    Formatter.instance().format_info("当前版本:"+ __version__)
    checkopts()

if __name__ == "__main__":
    sys.exit(checkopts())





#!/usr/bin/env python
# -*- coding=utf-8 -*-


class Formatter(object):
    """
    格式化类
    """

    __instance = None

    @classmethod
    def instance(cls):
        if cls.__instance is None:
            cls.__instance = Formatter()
        return cls.__instance

    def format_print(self, message, separator="", repeatCunt=0):
        if not message:
            return
        if separator:
            print (separator) * repeatCunt
        print str(message)
        if separator:
            print (separator) * repeatCunt + '\n'

    def format_error(self, message):
        """
        打印错误
        :param message: 消息
        """
        self.format_print(message, " *", 40)

    def format_info(self, message):
        """
        打印有站位符的普通消息
        :param message: 消息
        """
        self.format_print(message, " -", 40)

    def format_warning(self, message):
        """
        打印警告
        :param message: 消息
        """
        self.format_print(message, " ?", 40)



#!/usr/bin/env python
# -*- coding=utf-8 -*-


import sys
from formatter import Formatter

class Result(object):

    __instance = None

    def __init__(self):
        self.formatter = Formatter.instance()

    @classmethod
    def instance(cls):
        if cls.__instance is None:
            cls.__instance = Result()
        return cls.__instance

    def returnError(self, message=""):
        # TODO 清理工作
        self.formatter.format_error(message)
        exit(1)

    def returnSuccess(self, message=""):
        # TODO 清理工作。
        self.formatter.format_info(message)
        exit(0)




#!/usr/bin/env python
# -*- coding=utf-8 -*-


import sys
import os
import importlib
from Tools.tips import Tips
from Tools.result import Result
from Tools.formatter import Formatter
from Tools.fileHandle import FileHandle
from Tools.shellCommand import Shell
from Tools.arguments import Arguments
from Tools.cocoapodsTool import CocoapodsTool
from Tools.constant import *
from initSpec import InitSpec
from Tools.resultModel import ResultModel


class Check(object):
    __instance = None


    def __init__(self):
        self.formatter = Formatter.instance()
        self.result = Result.instance()
        self.tips = Tips.instance()
        self.fileHandle = FileHandle.instance()
        self.shell = Shell.instance()
        self.arguments = Arguments.instance()
        self.cocoaPodsTool = CocoapodsTool.instance()
        self.podspec = InitSpec.instance()

    @classmethod
    def instance(cls):
        if cls.__instance is None:
            cls.__instance = Check()
        return cls.__instance


    def check(self):
        os.chdir(self.arguments.projectPath)
        self.arguments.branch, self.arguments.commitId = self.gitTool.currentBranchAndCommitID()
        self.libLint()

    def libLint(self):
        self.cocoaPodsTool.libLint()
        ResultModel.instance().resultDict["checkSuccess"] = True
        self.formatter.format_info("组件check通过")


#!/usr/bin/env python
# -*- coding=utf-8 -*-


from Tools.formatter import Formatter
from Tools.shellCommand import Shell



class Env(object):
    __instance = None

    def __init__(self):
        self.formatter = Formatter.instance()
        self.shell = Shell.instance()

    @classmethod
    def instance(cls):
        if cls.__instance is None:
            cls.__instance = Env()
        return cls.__instance

    def runtimeEnv(self):
        """
        检查运行环境是否满足
        主要是检查非Python环境的
        :return:
        """
        returnCode, content = self.shell.excommand_until_done("which pip")
        if returnCode > 0:
            self.formatter.format_info("正在安装pip组件（一个类似于gem的安装器")
            self.shell.excommand_until_done("sudo -H easy_install pip")
        returnCode, content = self.shell.excommand_until_done("gem list | grep cocoapods-packager")
        if returnCode > 0:
            self.formatter.format_info("正在安装打包脚本cocoapods-packager, 如果需要改版的packager，请手动安装")
            # 本地的packager
            self.shell.excommand_until_done("sudo -H gem install cocoapods-packager")




#!/usr/bin/env python
# -*- coding=utf-8 -*-


import sys
import markdown
from markdown.extensions.wikilinks import WikiLinkExtension
from shellCommand import Shell

sys.getdefaultencoding()

class MDFormatter(object):
    """
    把输出转成markdown文件
    """
    __instance = None

    def __init__(self):
        self.shell = Shell.instance()

    @classmethod
    def instance(cls):
        if cls.__instance is None:
            cls.__instance = MDFormatter()
        return cls.__instance

    def title(self, title, level=1):
        return "\n" + "#" * level + " " + title + "\n"

    def code(self, code, type="Ruby"):
        """
        把code格式化成markdown的code
        :param code: 源码
        :type code str
        :param type:
        :return:
        """
        content = "\n"
        content += "``` " +  type + "\n"
        content += code
        content += "```"
        content += "\n"
        return content

    def table(self, tablelist, headelist=[]):
        """
        markdown 的表格写法
        :param tablelist:
        :param headelist:
        :return:
        """
        content = "\n"
        if len(headelist) > 0:
            content += "| " + " | ".join(headelist) + " |\n"
        separateList = [":------------:"] * len(headelist)
        content += "|" + " | ".join(separateList) + " |\n"
        # horizontal - vertical
        for hor_list in tablelist:
            content += "|" + " | ".join(hor_list) + " |\n"
        content += "\n"
        return content

    def orderListString(self, list):
        """
        有序列表字符串
        :param list: 列表
        :return:
        """
        content = "\n"
        index = 1
        for line in list:
            content += index + ". " + str(line) + "\n"
            index += 1
        content += "\n"
        return content

    def disorderListString(self, list):
        """
        无序列表
        :param list: 列表
        :return:
        """
        content = "\n"
        for line in list:
            content += "* " + str(line) + "\n"
        content += "\n"
        return content

    def to_html(self, content):
        css = self.css()
        html = markdown.markdown(content, output_format='html5', \
                                 extensions=['markdown.extensions.toc', \
                                             WikiLinkExtension(base_url='https://en.wikipedia.org/wiki/', \
                                                               end_url='#Hyperlinks_in_wikis'), \
                                             'markdown.extensions.sane_lists', \
                                             'markdown.extensions.codehilite', \
                                             'markdown.extensions.abbr', \
                                             'markdown.extensions.attr_list', \
                                             'markdown.extensions.def_list', \
                                             'markdown.extensions.fenced_code', \
                                             'markdown.extensions.footnotes', \
                                             'markdown.extensions.smart_strong', \
                                             'markdown.extensions.meta', \
                                             'markdown.extensions.nl2br', \
                                             'markdown.extensions.tables'])
        html = html.replace("<table>", "<table class=\"table\">")
        return css + html


    def url(self, url, message="链接"):
        """
        markdown 的url
        :param url: 网址
        :param message: 提示信息
        :return:
        """
        return "[" + message + "](" + url + ")"

    def image(self, url, tips=""):
        """
        ![MacDown logo](http://macdown.uranusjr.com/static/images/logo-160.png)
        :param url:
        :return:
        """
        return "![" + tips + "](" + url + ")"

    def bold(self, text):
        """
        粗体
        :param text:
        :return:
        """
        return "**" + text + "**"

    def italic(self, text):
        """
        斜体
        :param text:
        :return:
        """
        return "*" + text + "*"

    def css(self):
        return """
        <style>
    .table table {
        width:100%;
        margin:15px 0;
        border:0;
    }
    .table th {
        background-color:#B8D7FF;
        color:#000000
    }
    .table,.table th,.table td {
        font-size:0.95em;
        text-align:center;
        padding:4px;
        border-collapse:collapse;
    }
    .table th,.table td {
        border: 1px solid #a0c9fe;
        border-width:1px 0 1px 0;
        border:2px outset #ffffff;
    }
    .table tr {
        border: 1px solid #ffffff;
    }
    .table tr:nth-child(odd){
        background-color:#e7f1fe;
    }
    .table tr:nth-child(even){
        background-color:#ffffff;
    }
    </style>
    
        """


#!/usr/bin/env python
# -*- coding=utf-8 -*-


import sys
import os
from Tools.result import Result
from Tools.formatter import Formatter
from Tools.fileHandle import FileHandle
from Tools.shellCommand import Shell
from Tools.arguments import Arguments
from Tools.constant import *
from Tools.commonFuncs import *


class InitSpec(object):
    __instance = None

    def __init__(self):
        self.formatter = Formatter.instance()
        self.result = Result.instance()
        self.fileHandle = FileHandle.instance()
        self.shell = Shell.instance()
        self.arguments = Arguments.instance()

    @classmethod
    def instance(cls):
        if cls.__instance is None:
            cls.__instance = InitSpec()
        return cls.__instance

    # TODO 重复修改问题
    def supportBinary(self):
        if not self.fileHandle.file_exist(Arguments.instance().projectPath):
            self.result.returnError("没找到工程目录")
        os.chdir(self.arguments.projectPath)
        projectPath = self.arguments.projectPath
        podName = self.arguments.podName
        subPackage = self.arguments.subPackage
        originSpecPath = projectPath + "/" + podName + ".podspec"
        content = self.fileHandle.readFile(originSpecPath)
        sourceList = matchList("(s.source_files += +[\'|\"].*?[\'|\"])", content)
        publicHeaderList = matchList("(s.public_header_files += +[\"|\'].*?[\"|\'])", content)
        replaceContent = ""
        if sourceList:
            if publicHeaderList:
                replaceContent = content.replace(publicHeaderList[0], "", 1)
            sourceKey = sourceList[0]
            self.formatter.format_info("即将替换" + sourceKey)
            if subPackage:
                replaceContent = replaceContent.replace(sourceKey, packageBySubspec, 1)
            else:
                replaceContent = replaceContent.replace(sourceKey, noSubSpec, 1)
        else:
            self.result.returnError("错误，没有找到关键字source_files ，可能已经完成替换或者其他的未知错误")
        self.fileHandle.writeToFile(replaceContent, originSpecPath)
        self.formatter.format_info(podName + "支持源码和二进制切换成功：")




#!/usr/bin/env python
# -*- coding=utf-8 -*-

import urllib
from formatter import Formatter
from commonFuncs import *

class URL(object):
    __instance = None

    def __init__(self):
        self.formatter = Formatter.instance()

    @classmethod
    def instance(cls):
        if cls.__instance is None:
            cls.__instance = URL()
        return cls.__instance

    def isURL(self, URL):
        """
        判断URL是否合法
        :param URL: 提供的URL
        :return: 是否合法
        """
        legalURL = matchList(
            r"^(?:([A-Za-z]+):)?(\/{0,3})([0-9.\-A-Za-z]+)(?::(\d+))?(?:\/([^?#]*))?(?:\?([^#]*))?(?:#(.*))?$", URL)
        if legalURL:
            return True
        else:
            return False

    def URLProtocol(self, URL):
        # http://www.ahlinux.com:8001/test
        protocol, s1 = urllib.splittype(URL)
        # ('http', '//www.ahlinux.com:8001/test')
        return protocol, s1

    def URLPath(self, URL):
        protocol, s1 = self.URLProtocol(URL)
        hostAndPort, path = urllib.splithost(s1)
        # ('www.ahlinux.com:8001', '/test')
        return hostAndPort, path

    def URLHostAndPort(self,URL):
        # type: () -> (str, str)
        hostAndPort, path = self.URLPath(URL)
        host, port = urllib.splitport(hostAndPort)
        # ('www.ahlinux.com', '8001')
        return host, port

    def getUrl(self, content=""):
        content = content.replace("\'","").replace("\"","")
        list = matchList("[a-zA-z]+://[^\s]*", content)
        return list


#!/usr/bin/env python
# -*- coding=utf-8 -*-

from result import Result
import re


def boolValue(arg):
    value = False
    if arg in ("false", "FALSE", "False", "0"):
        value = False
    else:
        value = True
    return value

def matchList(pattern, string, byLine=False):
    """正则匹配

    Arguments:
        pattern {str} -- 正则表达式
        string {str} -- 要匹配的全量字符串

    Keyword Arguments:
        byLine {bool} -- 是不是一行一样匹配 (default: {False})

    Returns:
        [list] -- 匹配成功的列表
    """
    if string == "未知消息类型":
        Result.instance().returnError("没有匹配到相应的内容")
    if byLine:
        pattern = re.compile(pattern)
    else:
        pattern = re.compile(pattern, re.S)
    matchList = pattern.findall(string)
    if isinstance(matchList, list):
        return matchList
    else:
        Result.instance().returnError("没有匹配到相应的内容")


def byteify(input):
    # unicode 编码问题
    if isinstance(input, dict):
        return {byteify(key): byteify(value) for key, value in input.iteritems()}
    elif isinstance(input, list):
        return [byteify(element) for element in input]
    elif isinstance(input, unicode):
        return input.encode('utf-8')
    else:
        return input



#!/usr/bin/env python
# -*- coding=utf-8 -*-


import os
import importlib
import yaml
import re
import json
from constant import *
from arguments import Arguments
from formatter import Formatter
from result import Result
from commonFuncs import *
from tips import Tips
from fileHandle import FileHandle
from shellCommand import Shell
from url import URL

class PodspecTool(object):

    __instance = None

    def __init__(self):
        self.formatter = Formatter.instance()
        self.arguments = Arguments.instance()
        self.fileHandle = FileHandle.instance()
        self.result = Result.instance()
        self.shell = Shell.instance()
        self.urlTool = URL.instance()

    @classmethod
    def instance(cls):
        if cls.__instance is None:
            cls.__instance = PodspecTool()
        return cls.__instance

    def supportBinary(self):
        # changeSpec(podName, subPackage)
        projectPath = self.arguments.projectPath
        podName = self.arguments.podName
        subPackage = self.arguments.subPackage
        originSpecPath = projectPath + "/" + podName + ".podspec"
        content = self.fileHandle.readFile(originSpecPath)
        sourceList = matchList("(s.source_files += +[\'|\"].*?[\'|\"])", content)
        publicHeaderList = matchList("(s.public_header_files += +[\"|\'].*?[\"|\'])", content)
        replaceContent = ""
        if sourceList:
            if publicHeaderList:
                replaceContent = content.replace(publicHeaderList[0], "", 1)
            sourceKey = sourceList[0]
            self.formatter.format_print("即将替换" + sourceKey)
            if subPackage:
                replaceContent = replaceContent.replace(sourceKey, packageBySubspec, 1)
            else:
                replaceContent = replaceContent.replace(sourceKey, noSubSpec, 1)
        else:
            self.result.returnError("错误，没有找到关键字source_files ，可能已经完成替换或者其他的未知错误")
        self.fileHandle.writeToFile(replaceContent, originSpecPath)

    # TODO 不好。
    def changePodSpec(self, podName, subspecName="", version="", useTag=True, commitId="", dependencyDict={}, LFSURL="", replaceName=True, useSSH=False):
        podspecName = podName + subspecName
        originSpecPath = podName + ".podspec"
        if not self.fileHandle.file_exist(originSpecPath):
            self.result.returnError(originSpecPath + "文件不存在，请检查工程名或者podName是否正确")
        # content = self.fileHandle.readFile(originSpecPath)
        contentList = []
        for lineStr in self.fileHandle.readLines(originSpecPath):
            tmpLineStr = lineStr.replace("\t", "").strip(" ")
            if str(tmpLineStr).startswith("#"):
                contentList.append(lineStr)
                continue
            tmpLineStr = tmpLineStr.replace("\n", "")
            if str(tmpLineStr).startswith("s.name") and tmpLineStr.find(podName) > -1:
                originLine = tmpLineStr
                replaceSpecName = tmpLineStr.replace(podName, podspecName, 1)
                self.formatter.format_info("把" + lineStr + "替换成" + replaceSpecName)
                lineStr = lineStr.replace(originLine, replaceSpecName)
            elif str(tmpLineStr).startswith("s.version"):
                originLine = tmpLineStr
                if version:
                    replaceVersion = "s.version          = \'" + version + "\'"
                    self.formatter.format_info("把" + lineStr + "替换成" + replaceVersion)
                    lineStr = lineStr.replace(originLine, replaceVersion)
            elif tmpLineStr.startswith("s.source") and tmpLineStr.find(":git") > -1:
                originSourceLine = tmpLineStr
                replaceSource = tmpLineStr
                if useSSH:
                    urlList = self.urlTool.getUrl(tmpLineStr)
                    if urlList and len(urlList) > 0:
                        url = urlList[0]
                        urlProtol, s1 = self.urlTool.URLProtocol(url)
                        host, port = self.urlTool.URLHostAndPort(url)
                        replaceSource = replaceSource.replace(urlProtol, "ssh", 1).replace(host, self.arguments.user + "@" + host + ":29418")
                replaceSource = str(replaceSource).strip("\n").strip("").split(",")[0].strip("}")
                if LFSURL:
                    # if not self.urlTool.isURL(LFSURL):
                    #     self.result.returnError("提供的LFSURL不是格式不规范，请检查：" + LFSURL)
                    # 如果存在远程大文件存储服务器，pushrepo的时候，需要改成服务器下载地址
                    replaceSource = "s.source = { :http => \"" + LFSURL + "\""
                if commitId:
                    commitId = commitId.strip("\n")
                    if tmpLineStr.find(":commit") > -1 or tmpLineStr.find(":tag"):
                        replaceSource += ", :commit => \"" + commitId + "\" }"
                elif useTag:
                    replaceSource += ", :tag => s.name.to_s + \"-\" + s.version.to_s }"
                else:
                    replaceSource += " }"

                if replaceSource:
                    self.formatter.format_info("把" + tmpLineStr + "替换成" + replaceSource)
                    lineStr = lineStr.replace(originSourceLine, replaceSource)
            elif tmpLineStr.find(":dependency =>") > -1:
                originLine = tmpLineStr
                propertyLine = str(tmpLineStr).split(":dependency =>")[-1].replace("[", "").replace("{", ""). \
                    replace("}", "").replace("]", "")
                nameList = matchList(r":name +=> +\"(\w+?)\"", propertyLine)
                if nameList and nameList[0] in dependencyDict.keys():
                    specName = nameList[0]
                    versionList = matchList(r":version +=> +\"(.+?)\"", propertyLine)
                    if len(versionList) == 1:
                        if not isinstance(dependencyDict[specName], list):
                            self.result.returnError("给的字典值必须是数组")
                        replaceLine = dependencyDict[specName][-1]
                        self.formatter.format_info("把依赖：" + versionList[0] + "\n变成：   " + replaceLine + "\n")
                        lineStr = lineStr.replace(versionList[0], replaceLine)
            elif tmpLineStr.find("#{s.name}") > -1 and replaceName:
                originLine = tmpLineStr
                replaceLine = tmpLineStr.replace("#{s.name}", podName)
                self.formatter.format_info("把库名：" + tmpLineStr + "\n变成：   " + replaceLine + "\n")
                lineStr = lineStr.replace(originLine, replaceLine)
            elif tmpLineStr.find(".dependency ") > -1:
                originLine = tmpLineStr
                specName = tmpLineStr.split(" ")[-1].strip("\"").strip("\'").split("/")[0]
                if specName in dependencyDict.keys() and str.isalnum(specName[0]):
                    if not isinstance(dependencyDict[specName], list):
                        self.result.returnError("给的字典值必须是数组")
                    versionString = ""
                    for version in dependencyDict[specName]:
                        versionString += ", \"" + version + "\""
                    replaceLine = tmpLineStr + versionString
                    self.formatter.format_info("把" + lineStr + "替换成" + replaceLine)
                    lineStr = lineStr.replace(originLine, replaceLine)
            contentList.append(lineStr)
        # 去掉最后一个end
        endString = ""
        while (endString != "end"):
            endString = contentList.pop().strip("\n").strip("")
            print endString
        for key, versionList in dependencyDict.items():
            versionString = ""
            for vString in versionList:
                versionString += "\"" + vString + "\", "
            versionString = versionString.strip(" ").rstrip(",")
            dependencyString = "  s.dependency  \"" + str(key) + "\""
            if versionString:
                dependencyString += ", " + versionString
            contentList.append(dependencyString + "\n")
            self.formatter.format_info("添加依赖：" + dependencyString)
        contentList.append("end")
        replaceContent = "".join(contentList)
        self.fileHandle.writeToFile(replaceContent, podspecName + ".podspec")

    def changePodSpecForTmpPodspec(self, subSpecName):
        podName = self.arguments.podName
        dependencyDict = self.arguments.dependencyDict
        commitID = self.arguments.commitId
        version = self.arguments.version.split("-")[0]
        self.changePodSpec(podName + subSpecName, "Tmp", version, False, commitID, dependencyDict, "" , True, False)


    def changePodSpecForCheck(self):
        podName = self.arguments.podName
        dependencyDict = self.arguments.dependencyDict
        commitID = self.arguments.commitId
        version = self.arguments.version
        self.changePodSpec(podName, "", version ,False, commitID, dependencyDict)

    def changePodSpecForPackage(self, subspecName):
        podName = self.arguments.podName
        dependencyDict = self.arguments.dependencyDict
        commitId = self.arguments.commitId
        version = self.arguments.version
        self.changePodSpec(podName, subspecName, version,False, commitId, dependencyDict, "", True, self.arguments.useSSH)

    def changePodspecForPush(self):
        podName = self.arguments.podName
        version = self.arguments.version
        self.changePodSpec(podName, "", version, True, "", {}, "", False)

    def changePodspecForPushRepo(self):
        podName = self.arguments.podName
        LFSURL = self.arguments.LFSURL
        useTag = True
        if len(LFSURL):
            useTag = False
        version = self.arguments.version
        self.changePodSpec(podName, "", version, useTag, "", {}, LFSURL, False)



    def subspecList(self):
        """
        获得subspec的列表
        :param podName: pod库名称
        :return:
        """
        podName = self.arguments.podName
        podspecDict = self.getSpecDict(podName)
        subspecNameList = []
        if "subspecs" not in podspecDict.keys():
            self.result.returnError("没有subspec要打包，如果不需要每个subspec单独打包，请把subPackage设置成false")
        if podspecDict["subspecs"]:
            for subspec in podspecDict["subspecs"]:
                subspecNameList.append(subspec["name"])
        return byteify(subspecNameList)

    def getSpecDict(self, podName):
        returnCode, content = self.shell.excommand_until_done("IS_SOURCE=1 pod ipc spec " + podName + ".podspec")
        contentList = matchList("({.+}$)", content)
        if returnCode > 0 or not contentList:
            self.result.returnError("podspec 有问题，不能转换成JSON数据")
        specJson = contentList[0]
        specDict = json.loads(specJson)
        return byteify(specDict)

    def canPackage(self, podName, subspecName):
        specDict = self.getSpecDict(podName)
        sourcePathList = []
        if not subspecName:
            if "source_files" in specDict.keys():
                sourcePath = specDict["source_files"]
                if isinstance(sourcePath, str):
                    sourcePath = [sourcePath]
                sourcePathList.extend(sourcePath)
            if "subspecs" in specDict.keys():
                subspecs = specDict["subspecs"]
                for subspec in subspecs:
                    if "source_files" in subspec.keys():
                        sourcePath = subspec["source_files"]
                        if isinstance(sourcePath, str):
                            sourcePath = [sourcePath]
                        sourcePathList.extend(sourcePath)
        else:
            if "subspecs" in specDict.keys():
                subspecs = specDict["subspecs"]
                for subspec in subspecs:
                    if "name" in subspec.keys():
                        if subspecName != subspec["name"]:
                            continue
                    if "source_files" in subspec.keys():
                        sourcePath = subspec["source_files"]
                        if isinstance(sourcePath, str):
                            sourcePath = [sourcePath]
                        sourcePathList.extend(sourcePath)

        for sourcePath in sourcePathList:
            splitString = ""
            if "/**/*" in sourcePath:
                splitString = "/**/*"
            elif "/**/" in sourcePath:
                splitString = "/**/"
            elif "/*" in sourcePath:
                splitString = "/*"
            else:
                splitString = "/"
            tmpList = sourcePath.split(splitString)
            intersection = ["m", "mm", "c", "cpp", "cc"]
            if len(tmpList) >= 2:
                if tmpList[-1] != "":
                    suffixList = tmpList[-1].strip("{").strip("}").replace(".", "").split(",")
                    usefulSuffixList = ["m", "mm", "c", "cpp", "cc"]
                    intersection = [val for val in suffixList if val in usefulSuffixList]
                    if not intersection:
                        continue
                    sourcePath = sourcePath.rstrip(tmpList[-1])
            sourcePath = sourcePath.rstrip(splitString)
            for suffix in intersection:
                files = self.findfiles(sourcePath, "." + suffix, True)
                if not files:
                    continue
                return True
        return False

    def findfiles(self, dirname, suffix, recursion=False):
        result = []
        for fileName in os.listdir(dirname):
            filePath = os.path.join(dirname, fileName)
            if os.path.isfile(filePath):
                if filePath.endswith(suffix):
                    result.append(filePath)
            elif recursion and os.path.isdir(filePath):
                result.extend(self.findfiles(filePath, suffix, recursion))
        return result

    def redirectSubSpec(self, originSpecPath):
        self.formatter.format_info(" 正在重定向subspec的dependency，为打包做准备")
        if not self.fileHandle.file_exist(originSpecPath):
            self.result.returnError(originSpecPath + "需要重定向的文件不存在，请检查工程名或者podName是否正确")
        content = self.fileHandle.readFile(originSpecPath)
        contentList = []
        for lineStr in self.fileHandle.readLines(originSpecPath):
            tmpLineStr = lineStr.strip(" ")
            if str(tmpLineStr).startswith("#"):
                contentList.append(lineStr)
                continue
            tmpLineStr = tmpLineStr.replace("\n", "")
            if tmpLineStr.startswith("ss.dependency ") > -1 and tmpLineStr.find("/#{dep[:spec_name]}") > -1:
                originLine = tmpLineStr
                replaceLine = "ss.dependency \"#{s.name}Tmp/#{dep[:spec_name]}\""
                self.formatter.format_info("把" + lineStr + "替换成" + replaceLine)
                lineStr = lineStr.replace(originLine, replaceLine)
            contentList.append(lineStr)
        replaceContent = "".join(contentList)
        # 把原podspec重命名
        self.fileHandle.writeToFile(replaceContent, originSpecPath)

    def backUpPodspec(self):
        originSpecName = self.arguments.podName + ".podspec"
        destinationSpecName = originSpecName + ".old"
        if self.fileHandle.file_exist(originSpecName) and not self.fileHandle.file_exist(destinationSpecName):
            self.shell.excommand_until_done("cp -rf " + originSpecName + " " + destinationSpecName)

    def podspecForPushHeaderRepo(self, podName, version, url):
        content = self.podspecForPush(podName, version, url)
        content = content.replace("ss.source_files = spec[:source_files]", "ss.source_files = spec[:public_header_files]")
        if "ss.source_files = spec[:public_header_files]" not in content:
            content = content.replace(",m,mm,c,cpp,cc", "").replace(",mm", "").replace(",m", "").replace(",cpp",
                                                                                                                 "").replace(
                ",cc", "").replace(",c", "")
            content = re.sub("\.m\"", ".h\"", content, 0, flags=re.I)
            content = re.sub("\.m\'", ".h\'", content, 0, flags=re.I)
            if ".*" in content:
                content = re.sub("\.\*", ".h", content, 0, flags=re.I)
            if "/*\'" in content:
                content = re.sub("/\*\'", "/*.h\'", content, 0, flags=re.I)
            if "/*\"" in content:
                content = re.sub("/\*\"", "/*.h\"", content, 0, flags=re.I)
        content = re.sub("\t", "    ", content, 0, flags=re.I)
        if ".resource" in content:
            new_content = re.sub("end\n* *$", "    s.resources = nil\nend", content, 1, flags=re.I)
        if ".resource_bundle" in content:
            new_content = re.sub("end\n* *$", "    s.resource_bundles = nil\nend", content, 1, flags=re.I)
        content = re.sub("s.preserve_paths *=( *((\'|\")(\D*?)(\'|\") *,? *\n))*", "", content, 1,flags=re.I)
        content = re.sub("if spec\[:resource[s]?\]\n *ss.resources = spec\[:resource[s]?\]\n *end", "", content, 0,
                         flags=re.I)
        content = re.sub("if spec\[:resource_bundles?\]\n *ss.resource_bundles? = spec\[:resource_bundles?\]\n *end",
                         "", content, 0, flags=re.I)
        return content

    def podspecForPush(self, podName, version, url=""):
        podspec_name = podName + ".podspec"
        if not self.fileHandle.file_exist(podspec_name):
            self.result.returnError("找不到文件" + podspec_name)
        content = self.fileHandle.readFile(podspec_name)
        # 修改版本
        url = url.replace("http://git.lianjia.com", "http://gerrit.lianjia.com")
        content = re.sub("s.version *= *(\'|\")(\D|.)+?(\'|\")", "s.version         = \"%s\"" % (version), content,
                         1, flags=re.I)
        # 修改homePage
        content = re.sub("s.homepage *= *(\'|\")(\D*?)(\'|\")", "s.homepage         = \"%s\"" % (url), content, 1,
                         flags=re.I)
        # 修改url
        content = re.sub("s.source *= {.*}?",
                         "s.source           = { :git => \"%s\", :tag => s.name.to_s + \"-\" + s.version.to_s }" % (
                             url), content, 1, flags=re.I)
        return content

    def restorePodspec(self):
        destinationSpecName = self.arguments.podName + ".podspec"
        originSpecName = destinationSpecName + ".old"
        if self.fileHandle.file_exist(originSpecName):
            self.shell.excommand_until_done("cp -rf " + originSpecName + " " + destinationSpecName)
        self.deleteBackUpPodspec()

    def deleteBackUpPodspec(self):
        BackUpPodspec = self.arguments.podName + ".podspec" + ".old"
        if self.fileHandle.file_exist(BackUpPodspec):
            self.shell.excommand_until_done("rm " + BackUpPodspec)


#!/usr/bin/env python
# -*- coding=utf-8 -*-

import os
import time
import sys
import glob
import fcntl
import yaml
from formatter import Formatter


class FileHandle(object):

    __instance = None

    def __init__(self):
        self.formater = Formatter.instance()

    @classmethod
    def instance(cls):
        if cls.__instance is None:
            cls.__instance = FileHandle()
        return cls.__instance

    def newFileName(self, fileName, suffix):
        """文件重命名
        Arguments:
            fileName {string} -- 文件名
        Returns:
            string -- 重命名后的文件名
        """
        if '/' in fileName:
            return "_".join(fileName.split("/")) + suffix
        else:
            return fileName + suffix


    def newCookieFileName(self, webServer, suffix='.cookie'):
        """
        返回webServer的cookie名字
        Arguments:
            webServer {str} -- 访问的webServer
        Keyword Arguments:
            suffix {str} -- 后缀 (default: {'.cookie'})
        Returns:
            [str] -- 新的cookie名字
        """
        return self.newFileName(webServer, suffix)

    def newCacheFileName(self, fileName, suffix='.cache'):
        """
        返回缓存文件的名字
        Arguments:
            fileName {string} -- 原文件名
        Returns:
            string -- 缓存文件名
        """
        return self.newFileName(fileName, suffix)

    def newLogFileName(self, fileName, suffix='_diff.log'):
        """
        获取log文件的文件名
        Arguments:
            fileName {string} -- log文件名
        Returns:
            string -- [description]
        """
        return self.newFileName(fileName, suffix)


    def urlCacheFilePath(self, url, suffix='.cache'):
        """
        获取url缓存后的路径
        Arguments:
            url {string} -- url

        Returns:
            string -- 文件名
        """
        cacheFileName = self.newCacheFileName(url, suffix)
        filePath = self.file_path + "/" + cacheFileName
        return filePath

    def getFileCreateTime(self, filePath):
        """
        获取文件创建时间
        Arguments:
            filePath {str} -- 文件路径
        Returns:
            int -- 时间戳
        """
        filePath = unicode(filePath, 'utf8')
        t = os.path.getctime(filePath)
        return int(t)

    def getCurrentTime(self, filePath):
        "返回当前时间戳"
        return int(time.time())

    def versionHistoryPath(self, podName, url, LJurlPrefix):
        """返回更新文档的位置
        Arguments:
            podName {str} -- 库名
            url {str} -- 库的git地址
            LJurlPrefix {str} -- 链家库的前缀
        Returns:
            str -- 更新文档路径
        """

        versionHistoryPath = url
        if url.startswith(LJurlPrefix):
            # 是链家的
            versionHistoryPath += "/raw/master/"
            if url.split('/')[-1] != podName:
                versionHistoryPath += podName
            versionHistoryPath += "/VersionHistory.md"
        else:
            # 不是链家的
            versionHistoryPath += "/releases"
        return versionHistoryPath

    def fileOverDue(self, filePath, overTime):
        """
        判断文件是否过期
        Arguments:
            filePath {str} -- 文件路径
            overTime {int} -- 过期时间
        Returns:
            bool -- 是否过期
        """
        if self.getCurrentTime(filePath) - self.getFileCreateTime(filePath) - overTime >= 0:
            return True
        else:
            return False

    def readLines(self, filePath):
        """[按行读取文件的全部内容，返回数组]

        Arguments:
            filePath {string} -- 文件名

        Returns:
            List -- 每一行的数组
        """
        with open(filePath, "r") as file:
            return file.readlines()

    def isfile(self, fileName):
        """
        判断当前工程目录下有没有这个文件
        Arguments:
            fileName {String} -- fileName
        Returns:
            bool -- 文件是否存在
        """
        if os.path.isfile(fileName):
            return True
        else:
            return False

    def isDir(self, fileName):
        """
        判断是不是目录
        Arguments:
            fileName {String} -- fileName
        Returns:
            bool -- 文件是否存在
        """
        if os.path.isdir(fileName):
            return True
        else:
            return False

    def file_exist(self, filePath):
        """
        判断文件是否存在
        :param filePath: 文件目录
        :return:
        """
        if os.path.exists(filePath):
            return True
        else:
            return False

    def delTempFiles(self, fileList):
        """删除临时文件
        Arguments:
            fileList {List} -- 临时文件列表
        """
        self.formater.format_print("临时文件列表为：" + str(fileList))
        for fileName in fileList:
            logFilePath = fileName
            if os.path.isfile(logFilePath):
                self.formater.format_print('正在删除临时文件' + logFilePath)
                os.remove(logFilePath)
            elif os.path.isdir(logFilePath):
               self.formater.format_print('文件夹暂时不允许删除')

    def writeToFile(self, string, filePath):
        """
        把内容写进文件,线程安全
        Arguments:
            string {string} -- 文件内容
            filePath {string} -- 文件路径
        """
        with open(filePath, "w+") as fileWriter:
            fcntl.flock(fileWriter, fcntl.LOCK_EX)
            fileWriter.write(string)
            fileWriter.flush()
            fcntl.flock(fileWriter, fcntl.LOCK_UN)

    def appendToFile(self, string, filePath):
        with open(filePath, "a+") as fileWriter:
            fileWriter.write(str(string))

    def writeYamlToFile(self, string, filePath):
        """
        把 内容写进yaml文件
        :param string: 文件内容
        :param filePath: 文件路径
        :return:
        """
        with open(filePath, "w+") as fileWriter:
            fcntl.flock(fileWriter, fcntl.LOCK_EX)
            yaml.dump(string, fileWriter)
            fcntl.flock(fileWriter, fcntl.LOCK_UN)

    def readFile(self, filePath):
        """
        读取文件内容
        :param filePath: 文件路径
        :return:
        """
        with open(filePath, "r+") as fileReader:
            return fileReader.read()

    def readYamlFile(self, filePath):
        """
        读取yaml文件
        :param filePath: 文件路径
        :return:
        """
        with open(filePath, "r+") as fileReader:
            return yaml.load(fileReader.read())

    def removeFile(self, filePath):
        if self.file_exist(filePath):
            try:
                os.remove(filePath)
            except IOError:
                print ('系统错误，无法删除文件-' + filePath + '，可能被占用')
                # 清空
                self.writeToFile("", filePath)

    def myGlob(self, path, recursion=False):
        fileList = []
        for fileName in os.listdir(path):
            fileDir = os.path.join(path, fileName)
            # nestBasename = os.path.join(fileDir, fileName)
            if os.path.isdir(fileDir):
                if recursion:
                    nextFileList = self.myGlob(fileDir, recursion)
                    if nextFileList:
                        fileList.extend(nextFileList)
            else:
                fileList.append(fileDir)
        return list(set(fileList))




#!/usr/bin/env python
# -*- coding=utf-8 -*-


"""
初始化模板的的时候全局变量
"""
emailSuffix = "@ke.com"
platform = "iOS"
language = "ObjC"
demo = "Yes"
testing = "None"
based = "No"
prefix = "LJ"


"""
标准模板示例Example中需要忽略的内容
"""
ignoreFileString = """

Pods/
Example/Pods/
.config/cafswitcher_config.yml

"""

# "提交脚本，即review.sh  想要提交的时候，主要在命令行运行： sh review.sh 就能提交gerrit"
reviewInfo = """

#bin/bash

git_prefix=".git"

install_commit_msg(){ 
    if [ ! -f ".git/hooks/commit-msg" ]; then
        echo "请输入用户名(不需要加后缀)"
        read username
        gitdir=$(git rev-parse --git-dir); 
        scp -p -P 29418 ${username}@gerrit.lianjia.com:hooks/commit-msg ${gitdir}/hooks/
        if [ ! $? -eq 0 ]; then
            echo "commit-msg下载错误"
            exit 1
        else
            echo "已经存在"
        fi
    fi
}

if [ ! -d "$git_prefix" ]; then
	echo "! [Illegal git repository directory]"
	echo "  移动脚本到git仓库根目录"
	exit 1
fi


if [ ! -d ".git/hooks" ]; then
    mkdir ".git/hooks"
	echo "mkdir successfull"
fi

while getopts "m:c" arg
do
	case $arg in
		m)
		  echo "git commit -a -m ..."
          install_commit_msg
          git commit -a -m "$OPTARG"
          ;;
		c)
		  echo "git commit -a --amend -C HEAD"
          install_commit_msg
          git commit -a --amend -C HEAD;
          ;;
	esac
done


if [ -f ".git/HEAD" ]; then
    head=$(< ".git/HEAD")
    if [[ $head = ref:\ refs/heads/* ]]; then
        git_branch="${head#*/*/}"
    else
        echo "无法获取当前分支"
	    exit 1
    fi

else
    echo "没有git中的HEAD文件"
	exit 1
fi


reviewers=("handa, zhangyansong, yuanyueguang001, zhaohongwei002, songhongri001, lixiangyu004, zhaoxiaomeng001")

echo "当前分支为:$git_branch"

pushUrl="HEAD:refs/for/$git_branch%"
for reviewer in ${reviewers[@]}; do 

    echo "reviewer人员为${reviewer}"    
    pushUrl="${pushUrl}r=${reviewer},"
done
pushUrl="${pushUrl%,*}"
echo "pushUrl为:$pushUrl"
git push origin $pushUrl
if [ $? -eq 0 ]; then
	exit 0
else
	exit 1
fi


"""

noSubSpec = """
    s.preserve_paths = "#{s.name}/Classes/**/*", "#{s.name}/Assets/**/*", "#{s.name}/Framework/**/*", "#{s.name}/Archive/**/*", "#{s.name}/Dependencies/**/*"

    configuration = "Debug"
    if ENV["IS_DEBUG"] || ENV["#{s.name}_DEBUG"]
      configuration = "Debug"
    elsif ENV["IS_RELEASE"] || ENV["#{s.name}_RELEASE"]
      configuration = "Release"
    end

    if ENV['IS_SOURCE'] || ENV["#{s.name}_SOURCE"]
      # 源码部分，请在这里写上必要的。
      s.source_files = "#{s.name}/Classes/**/*.{h,m,mm,c,cpp,cc}"
      s.public_header_files = "#{s.name}/Classes/**/*.h"
      # 如果有自己依赖的库，请写在这里，并且把依赖的.a 或者 .framework 放到Classes 同级别的Dependencies目录下
      # s.source_files = "#{s.name}/Classes/**/*.{h,m,mm,c,cpp,cc}", "#{s.name}/Dependencies/**/*.{h,m,mm,c,cpp,cc}"
      # s.public_header_files = "#{s.name}/Classes/**/*.h", "#{s.name}/Dependencies/**/*.h"
      # ...
    elsif ENV['IS_ARCHIVE'] || ENV["#{s.name}_ARCHIVE"]
      s.public_header_files = "#{s.name}/Archive/#{configuration}/*.h"
      s.source_files = "#{s.name}/Archive/#{configuration}/*.h"
      s.vendored_libraries = "#{s.name}/Archive/#{configuration}/lib#{s.name}.a"
      # 如果源码有依赖的库，上面这一行需要换成下面的
      # s.public_header_files = "#{s.name}/Archive/#{configuration}/*.h", "#{s.name}/Dependencies/**/*.h"
      # s.source_files = "#{s.name}/Archive/#{configuration}/*.h", "#{s.name}/Dependencies/**/*.h"
      # s.vendored_libraries = "#{s.name}/Archive/#{configuration}/lib#{s.name}.a","#{s.name}/Dependencies/**/*.a"
      # s.vendored_frameworks= "#{s.name}/Dependencies/**/*.framework"
    else
      s.public_header_files = "#{s.name}/Framework/#{configuration}/*.h"
      s.source_files = "#{s.name}/Framework/#{configuration}/*.h"
      s.vendored_frameworks = "#{s.name}/Framework/#{configuration}/#{s.name}.framework"
      # 如果源码有依赖的库，上面的一行酌情换成下面的语句
      # s.public_header_files = "#{s.name}/Framework/#{configuration}/*.h", "#{s.name}/Dependencies/**/*.h"
      # s.source_files = "#{s.name}/Framework/#{configuration}/*.h", "#{s.name}/Dependencies/**/*.h"
      # s.vendored_frameworks = "#{s.name}/Framework/#{configuration}/#{s.name}.framework","#{s.name}/Dependencies/**/*.framework"
      # s.vendored_libraries = "#{s.name}/Dependencies/**/*.a"
    end
    # 公共部分，比如公共资源类。framework等
    # s.resources        = "#{s.name}/Assets/**/*.{bundle,lic,png,jpg,plist,xib}"
    # 或者
    # s.resource_bundle = {
    #   "#{s.name}" => "#{s.name}/Assets/*.{storyboard,xib,json,xcassets,png,html}"
    # }

 """

packageBySubspec = """

  s.preserve_paths = "#{s.name}/Classes/**/*", "#{s.name}/Assets/**/*", "#{s.name}/Framework/**/*", "#{s.name}/Archive/**/*", "#{s.name}/Dependencies/**/*"

  # :spec_name => 名字
  _Core = {:spec_name => "Core"}
  # :sub_dependency => 内部依赖的数组, 变量是subspec形式的。
  # :resources => 指向资源的路径字符串
  _Foundation = {:spec_name => "Foundation", :resources => "#{s.name}/Assets/**/*.{bundle,lic,png,jpg,plist}" , :sub_dependency => [_Core]}
  # :dependency => 字典元素的数组， 里面每个字典包含两个元素 :name=> 第三方库名字， :version => 版本号(如果不指定，不写)
  _UIKit = {:spec_name => "UIKit", :dependency => [{:name => "AFNetworking", :version => "3.2.0"}]}

  #subspec 的集合
  _subspecs = [_Core, _Foundation, _UIKit]

  configuration = "Debug"
  if ENV["IS_DEBUG"] || ENV["#{s.name}_DEBUG"]
    configuration = "Debug"
  elsif ENV["IS_RELEASE"] || ENV["#{s.name}_RELEASE"]
    configuration = "Release"
  end

  _subspecs.each do |spec|
    if spec.delete(:noSource)
      next
    end
    if ENV["#{s.name}_#{spec[:spec_name]}_SOURCE"] || ENV['IS_SOURCE']
      #源码部分
      spec[:source_files] = "#{s.name}/Classes/#{spec[:spec_name]}/**/*.{h,m,mm,c,cpp,cc}"
      spec[:public_header_files] = "#{s.name}/Classes/#{spec[:spec_name]}/**/*.h"
    elsif ENV["#{s.name}_#{spec[:spec_name]}_ARCHIVE"] || ENV['IS_ARCHIVE']
      spec[:source_files] = "#{s.name}/Archive/#{spec[:spec_name]}/#{configuration}/*.h"
      spec[:public_header_files] = "#{s.name}/Archive/#{spec[:spec_name]}/#{configuration}/*.h"
      spec[:vendored_libraries] = "#{s.name}/Archive/#{spec[:spec_name]}/#{configuration}/*.a"
      # 如果源码有依赖的库，上面这一行需要换成下面的
      # s.public_header_files = "#{s.name}/Archive/#{configuration}/*.h", "#{s.name}/Dependencies/**/*.h"
      # s.source_files = "#{s.name}/Archive/#{configuration}/*.h", "#{s.name}/Dependencies/**/*.h"
      # s.vendored_libraries = "#{s.name}/Archive/#{configuration}/lib#{s.name}.a","#{s.name}/Dependencies/**/*.a"
      # s.vendored_frameworks= "#{s.name}/Dependencies/**/*.framework"
    else
      spec[:source_files] = "#{s.name}/Framework/#{spec[:spec_name]}/#{configuration}/*.h"
      spec[:public_header_files] = "#{s.name}/Framework/#{spec[:spec_name]}/#{configuration}/*.h"
      spec[:vendored_frameworks] = "#{s.name}/Framework/#{spec[:spec_name]}/#{configuration}/*.framework"
      # 有外部依赖的静态库的话请注释上面一行，换下面的
      # s.public_header_files = "#{s.name}/Framework/#{configuration}/*.h", "#{s.name}/Dependencies/**/*.h"
      # s.source_files = "#{s.name}/Framework/#{configuration}/*.h", "#{s.name}/Dependencies/**/*.h"
      # s.vendored_frameworks = "#{s.name}/Framework/#{configuration}/#{s.name}.framework","#{s.name}/Dependencies/**/*.framework"
      # s.vendored_libraries = "#{s.name}/Dependencies/**/*.a"
    end
  end

  _subspecs.each do |spec|
    s.subspec spec[:spec_name] do |ss|
      if spec[:source_files]
        ss.source_files = spec[:source_files]
      end

      if spec[:public_header_files]
        ss.public_header_files = spec[:public_header_files]
      end

      if spec[:vendored_libraries]
        ss.vendored_libraries = spec[:vendored_libraries]
      end

      if spec[:vendored_frameworks]
        ss.vendored_frameworks = spec[:vendored_frameworks]
      end

      if spec[:resources]
        ss.resources = spec[:resources]
      end

      if spec[:sub_dependency]
        spec[:sub_dependency].each do |dep|
          ss.dependency "#{s.name}/#{dep[:spec_name]}"
        end
      end

      if spec[:dependency]
        spec[:dependency].each do |dep|
          if dep.has_key?(:version)
            ss.dependency dep[:name], dep[:version]
          else
            ss.dependency dep[:name]
          end
        end
      end

      if spec[:frameworks]
        spec[:frameworks].each do |f|
          ss.framework = "#{f}"
        end
      end
    end
  end
  # 公共部分
  # ――― Resources ―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――― #
  #
  #  A list of resources included with the Pod. These are copied into the
  #  target bundle with a build phase script. Anything else will be cleaned.
  #  You can preserve files from being cleaned, please don't preserve
  #  non-essential files like tests, examples and documentation.
  #

  # s.resource  = "icon.png"
  # s.resources = "Resources/*.png"

  # ――― Project Linking ―――――――――――――――――――――――――――――――――――――――――――――――――――――――――― #
  #
  #  Link your library with frameworks, or libraries. Libraries do not include
  #  the lib prefix of their name.
  #
  # 公共部分，比如公共资源类。framework等
  # s.framework  = "SomeFramework"
  # s.frameworks = "SomeFramework", "AnotherFramework" 
  # s.library   = "iconv"
  # s.libraries = "iconv", "xml2" 
  # ――― Project Settings ――――――――――――――――――――――――――――――――――――――――――――――――――――――――― #
  #
  #  If your library depends on compiler flags you can set them in the xcconfig hash
  #  where they will only apply to your library. If you depend on other Podspecs
  #  you can include multiple dependencies to ensure it works.

"""


#!/usr/bin/env python
# -*- coding=utf-8 -*-


import sys
from arguments import Arguments
from formatter import Formatter
from shellCommand import Shell
from result import Result



class GitTool(object):

    __instance = None

    def __init__(self):
        # self.__load_config()
        self.arguments = Arguments.instance()
        self.formatter = Formatter.instance()
        self.shell = Shell.instance()
        self.result = Result.instance()

    @classmethod
    def instance(cls):
        if cls.__instance is None:
            cls.__instance = GitTool()
        return cls.__instance

    def _git(self, cmd, projectPath=""):
        if not projectPath:
            projectPath = self.arguments.projectPath
        projectGitPath = projectPath + "/.git"
        cmd = "git --work-tree=" + projectPath + " --git-dir=" + projectGitPath + " " + cmd
        self.formatter.format_print(cmd)
        returnCode, content = self.shell.excommand_until_done(cmd)
        if returnCode > 0:
            self.result.returnError("执行命令失败："+ cmd)
        return (returnCode, content)

    def clean(self, projectPath=""):
        self._git("add .", projectPath)
        if self.arguments.debug:
            return self._git("reset --soft HEAD~1", projectPath)
        else:
            return self._git("reset --hard", projectPath)

    def pull(self, projectPath=""):
        return self._git("pull", projectPath)

    def push(self, projectPath=""):
        return self._git("push origin ", projectPath)

    def commit(self, message, projectPath=""):
        self._git("add .", projectPath)
        return self._git("commit -m \"" + message + "\"", projectPath)

    def realBranchs(self, commitId):
        returnCode, content = self._git("branch -r --contains " + commitId)
        if returnCode > 0:
            self.result.returnError(" 请确认commtid是否正确")
        branchs = content.strip("\n").split("\n")
        branchList = []
        for branch in branchs:
            if "-> origin/master" in branch:
                continue
            branch = str(branch).strip("\n").strip(" ").replace("origin/", "")
            branchList.append(branch)
        return branchList

    def currentBranchAndCommitID(self):
        # 判断branch和commit
        commitId = self.arguments.commitId
        branch = self.arguments.branch
        projectPath = self.arguments.projectPath
        if commitId:
            realBranchList = self.gitTool.realBranchs(commitId)
            if branch and branch not in realBranchList:
                self.result.returnError("commit 所在分支并非提供的分支" + branch + "，而是在" + str(realBranchList))
            if not branch:
                if realBranchList and len(realBranchList) == 1:
                    realBranch = realBranchList[0]
                    self.formatter.format_info("即将用在commitID" + commitId + "所在的分支" + realBranch + "进行打包")
                    branch = realBranch
                else:
                    self.result.returnError("提供的commitID在如下的branch里：" + str(realBranchList) + ",请用--branch指定一个")
        else:
            if not branch:
                self.result.returnError("commitID 或者 branch 至少提供一种")
            else:
                self.formatter.format_info("即将用在分支" + branch + "上用最新的commit进行打包")
                count = self.unPushedCount()
                if count == 0:
                    cmd = "rev-parse " + branch
                else:
                    cmd = "rev-parse " + branch + "~" + str(count)
                returnCode, newCommitId = self._git(cmd, projectPath)
                if returnCode > 0:
                    self.result.returnError("不能获取当前分支的最新commit")
                commitId = str(newCommitId).strip("\n")
        return (branch, commitId)

    def unPushedCount(self, projectPath=""):
        returnCode, content = self._git("cherry", projectPath)
        count = 0
        if content:
            count = len(content.strip("").strip("\n").split("\n"))
        return count

    def checkout(self, branch, projectPath=""):
        return self._git("checkout " + branch, projectPath)

    def tag(self):
        return self._git("tag ")



#!/usr/bin/env python
# -*- coding=utf-8 -*-


from formatter import Formatter

class Tips(object):
    __instance = None

    def __init__(self):
        self.formatter = Formatter.instance()

    @classmethod
    def instance(cls):
        if cls.__instance is None:
            cls.__instance = Tips()
        return cls.__instance

    def showTips(self):
        tips = """脚本参数错误，请参考以下提示：

            需要先看看相关python环境有没有安装好。请运行
                > specCreator env
            参数：
                --user                  gerrit用户名
                --projectPath           工程地址
                --podName               pod库名（需要再根目录)
                --branch                在哪个分支上打包
                --version               要打包的版本
                --commitId              打包的时候依照哪个commit开始打包（这个commitId必须已经push到远程）  
                --subPackage            打包的时候是否要每个subspec单独打包。（默认为False)
                --moduleSources         组件池地址，即打包的source源，如果是多个，用逗号分开
                --updateMasterSource    是否更新master源，因为更新比较慢，默认不更新
                --dependencyJSON        依赖的库的JSON（字典）,可以是JSON文件的路径
                --moduleSources         组件池，不包括master源，master会内置。
                --updateMasterSource    Bool 值，是否更新master源，默认为false.
                --useSSH                打包的时候是否用SSH 下载，有时候提交太大，需要用SSH
                --check                 打包前要检查podspec书写是否规范
                --subspecs              字符串，需要打的subspec，多个用逗号隔开。前提是subPackage为True.（如果只提供部分subspec，其他的subspec不处理（即如果其他的subspec有静态库，不删除，如果没有，也不会重新打)，适用于做subspec的部分更新，而不需要把没变化的subspec也打静态库，节省时间）
                --framework             要打Framework静态库
                --debugPackage          打debug包
                --releasePackage        打release包
                --autoPush              打完包后自动commit并push
                --autoPushRepo          提交后自动更新repo发布。如果已经在repo白名单不需要此操作
                --repoSources           repo仓库的url，地址发布到哪个仓库
                --resultPath            必要信息会以JSON格式追加到这个文件
                --CAFPath               打包后源码和静态库存放的路径，如果不提供，则在工程目录下输出二进制结果
                --LFSURL                是否提供大文件存储的URL，如果提供，则会在发布repo的时候改变podspec 里s.source 里的URL



            本脚本的功能：
                1：从0开始创建一个spec标准库模板（暂不支持从一个已经存在的工程创建）
                    > specCreator init 
                        --ser=handa 
                        --projectPath=/Users/handa/Documents/LJABCD
                2：spec标准库检查。
                    > specCreator check
                        --user=handa
                        --projectPath=/Users/handa/Documents/LJCache 
                        --podName=LJCache 
                        --dependencyJSON="{\"LJHTTPService\":\"0.2.2\",\"AFNetworking\":\"3.2.1\"}"
                3：创建spec标准库的静态库（.framework)
                前提是：有spec工程，且工程里改了spec文件，并检查通过
                    > specCreator package
                        --user=handa
                        --projectPath=/Users/handa/Documents/LJCache
                        --podName=LJCache
                        --version=0.1.25
                        --commitId=ffa72511e6166222f294ddd874fd54247e82b369
                        --debug=false
                        --dependencyJSON="{\"LJHTTPService\":\"0.2.2\",\"AFNetworking\":\"3.2.1\"}"
                4：创建spec标准库的静态库（.a）（暂不支持生成非标准模板的静态库）
                前提是：有spec工程，且工程里改了spec文件，并检查通过
                    > specCreator packageA
                        --projectPath=/Users/handa/Documents/LJCache
                        --podName=LJCache
                        --version=0.1.25
                        --subPackage=false
                        --commitId=ffa72511e6166222f294ddd874fd54247e82b369
                        --debug=false
                        --dependencyJSON="{\"LJHTTPService\":\"0.2.2\",\"AFNetworking\":\"3.2.1\"}"
                5：对已经存在的podspec文件进行源码和静态库的适配。
                    > specCreator initSpec
                        --projectPath=/Users/handa/Documents/LJABCD
                        --podName=LJABCD

            对服务端如果完全自动化，请运行：
                    > specCreator autoPackage
                        --user=handa
                        --projectPath=/Users/handa/Documents/lianjia/LJCache
                        --podName=LJCache
                        --branch=develop
                        --subPackage=true
                        --version=0.2.3
                        --dependencyJSON="{\"AFNetworking\":[\"3.1.0\"]}"
                        --moduleSources=http://gerrit.lianjia.com/mobile_ios/LJComponentPodSpecs,http://gerrit.lianjia.com/mobile_ios/Lianjia_component_Podspec.git
                        --resultPath=/Users/handa/Documents/lianjia/LJCacheResult.json
                        --CAFPath=/Users/handa/Documents/lianjia/LJCacheCAFPath/
                        --framework
                        --debugPackage
                        --releasePackage
                        --useSSH
                        --autoPush
                        --autoPushRepo
                        --repoSource=http://gerrit.lianjia.com/mobile_ios/Lianjia_component_Podspec.git
                        --subspecs=Foundation,UIKit,Core

            init 运行前提：
                1：先从韩达那里申请创建一个girrit项目，从那里获取工程名，如：LJCache

            运行前提灰常重要
        """
        self.formatter.format_info(tips)


#!/usr/bin/env python
# -*- coding=utf-8 -*-

import os
import sys
import importlib
import glob
from Tools.tips import Tips
from Tools.result import Result
from Tools.formatter import Formatter
from Tools.fileHandle import FileHandle
from Tools.shellCommand import Shell
from Tools.arguments import Arguments
from Tools.cocoapodsTool import CocoapodsTool
from Tools.podspecTool import PodspecTool
from Tools.resultModel import ResultModel
from Tools.commonFuncs import *


class Package(object):
    __instance = None

    def __init__(self):
        self.formatter = Formatter.instance()
        self.result = Result.instance()
        self.tips = Tips.instance()
        self.fileHandle = FileHandle.instance()
        self.shell = Shell.instance()
        self.arguments = Arguments.instance()
        self.cocoaPodsTool = CocoapodsTool.instance()
        self.podspecTool = PodspecTool.instance()


    @classmethod
    def instance(cls):
        if cls.__instance is None:
            cls.__instance = Package()
        return cls.__instance

    def package(self):
        if not self.arguments.framework:
            return
        frameworkPath = os.path.join(self.arguments.podName, "/Framework")
        if self.arguments.debugPackage:
            self.formatter.format_info("正在打 framework debug 二进制库")
            self.packageFramework(True)
            ResultModel.instance().resultDict["debugFrameworkPath"] = os.path.join(frameworkPath, "Debug")
        if self.arguments.releasePackage:
            self.formatter.format_info("正在打 framework Release 二进制库")
            self.packageFramework(False)
            ResultModel.instance().resultDict["releaseFrameworkPath"] = os.path.join(frameworkPath, "Release")



    def packageFramework(self, debug):
        os.chdir(self.arguments.projectPath)
        subPackage = self.arguments.subPackage
        podName = self.arguments.podName
        contentDict = {}
        if subPackage:
            subspecNames = self.podspecTool.subspecList()
            ResultModel.instance().resultDict["subspecs"] =subspecNames
            for subspec in subspecNames:
                content = self.startPackage(subspec, debug)
                contentDict[subspec] = content
        else:
            if self.arguments.isflutter:
                content = self.flutterPackage("", debug)
            else:
                content = self.startPackage("", debug)
                contentDict[podName] = content
        if contentDict:
            dependency = self.processDependency(contentDict)
            ResultModel.instance().resultDict["dependency"] = dependency
            ResultModel.instance().versionHistory += "* dependency :" + str(dependency)

    def excommand(self, cmd):
        if cmd.startswith("flutter"):
            cmd = "export PUB_HOSTED_URL=https://pub.flutter-io.cn;export FLUTTER_STORAGE_BASE_URL=https://storage.flutter-io.cn;" + cmd
        returnCode, content = self.shell.excommand_until_done(cmd)
        if returnCode > 0:
            self.result.returnError("执行命令：" + str(cmd) + "发生错误，错误原因：" + str(content))
            return ""
        else:
            return content

    def flutterPackage(self, subSpecName, debug):
        # flutter clean
        self.excommand("flutter packages get")
        self.excommand("flutter packages upgrade")
        # 找到所有的plugin
        # pluginSource = "ios/pluginSource"
        pluginList = ""
        self.shell.excommand_until_done("flutter clean")
        # 清理现场
        self.shell.excommand_until_done("rm -rf build")
        self.shell.excommand_until_done("rm -rf combineDir")
        self.shell.excommand_until_done("mkdir -p combineDir/deviceApp")
        self.shell.excommand_until_done("mkdir -p combineDir/simulatorApp")
        self.shell.excommand_until_done("rm -rf ios/Flutter/App.framework ios/Flutter/Flutter.framework")
        self.shell.excommand_until_done("cd ios; pod update --verbose")
        if debug:
            flavercmd = ""
            build_device_path = "build/ios/Debug-iphoneos"
            build_iphonesimulator_path = "build/ios/Debug-iphonesimulator"
            if self.arguments.flaver:
                flavercmd = " --flavor " + self.arguments.flaver
                build_device_path = "build/ios/Debug-" + self.arguments.flaver +"-iphoneos"
                build_iphonesimulator_path = "build/ios/Debug-" + self.arguments.flaver +"-iphonesimulator"
            # 生成真机
            self.excommand("flutter build ios --debug --no-simulator --no-codesign" + flavercmd)
            self.excommand("cp ios/Flutter/App.framework/App combineDir/deviceApp")
            self.excommand("lipo -info combineDir/deviceApp/App")
            # pluginList = os.listdir(pluginSource)
            # self.copyPluginBinarys(pluginList, build_device_path, "combineDir/deviceApp")
            # 生成模拟器
            self.excommand("flutter build ios --debug --simulator --no-codesign" + flavercmd)
            self.excommand("cp ios/Flutter/App.framework/App combineDir/simulatorApp")
            self.excommand("lipo -info combineDir/simulatorApp/App")
            # pluginList = os.listdir(pluginSource)
            # self.copyPluginBinarys(pluginList, build_iphonesimulator_path, "combineDir/simulatorApp")
            # 合并 App
            self.excommand("lipo -create combineDir/simulatorApp/App combineDir/deviceApp/App -output ios/Flutter/App.framework/App")
            self.unionBinary(pluginList, "combineDir/deviceApp", "combineDir/simulatorApp", "combineDir")
        else:
            buildCMD = "flutter build ios --release --no-codesign"
            if self.arguments.flaver:
                buildCMD += " --flavor " + self.arguments.flaver
            # 产生APP.framework
            self.excommand(buildCMD)
            # pluginList = os.listdir(pluginSource)
        podName = self.arguments.podName
        productDir = os.path.join(self.arguments.projectPath, podName)
        CAFPath = self.arguments.CAFPath
        if CAFPath:
            if not self.fileHandle.file_exist(CAFPath):
                self.formatter.format_warning("存放二进制目录不存在，正在创建存放源码二进制的目录")
                self.excommand("mkdir -p " + CAFPath)
            productDir = os.path.join(CAFPath, podName)
        if self.arguments.framework:
            productDir = os.path.join(productDir, "Framework")
        productPath = ""
        if debug:
            productDir += "/Debug"
        else:
            productDir += "/Release"
        # 插件生成Framework
        # for plugin in pluginList:
        #     if plugin.startswith("."):
        #         continue
        #     # 生成framework
        #     binaryPath = "build/ios/Release-iphoneos/"+plugin + "/lib" + plugin + ".a"
        #     if self.arguments.flaver:
        #         binaryPath = "build/ios/Release-" + self.arguments.flaver + "-iphoneos/" + plugin + "/lib" + plugin + ".a"
        #     if debug:
        #         binaryPath = "combineDir/" + plugin
        #     self.createFramework(os.path.join(productDir, plugin+".framework"), plugin, binaryPath)
        os.chdir(self.arguments.projectPath)
        # 把APP.framework Flutter.framework 移动到指定位置。
        # if self.fileHandle.file_exist("ios/Flutter/Flutter.framework"):
        #     returnCode, content = self.shell.excommand_until_done("cp -R -p -f ios/Flutter/Flutter.framework " + productDir)
        #     if returnCode != 0:
        #         self.result.returnError("复制Flutter.framework失败:"+ content)
        if not self.fileHandle.file_exist(productDir):
            self.shell.excommand_until_done("mkdir -p " + productDir)
        if self.fileHandle.file_exist("ios/Flutter/App.framework"):
            returnCode, content = self.shell.excommand_until_done("cp -R -p -f ios/Flutter/App.framework " + productDir)
            if returnCode != 0:
                self.result.returnError("复制App.framework失败:"+ content)
        self.formatter.format_info("搞定")

    def copyPluginBinarys(self, pluginList, sourcePath, tagretPath):
        """
        复制插件到指定目录
        :param pluginList: 插件目录
        :param sourcePath: 原路径
        :param tagretPath: 目的路径
        """
        for plugin in pluginList:
            if plugin.startswith("."):
                continue
            sourceLibPath = sourcePath + "/" + plugin + "/lib" + plugin + ".a"
            if not self.fileHandle.file_exist(sourceLibPath):
                self.result.returnError(plugin + "没有找到")
            self.shell.excommand_until_done("cp -R -p " + sourceLibPath + " " + tagretPath)

    def unionBinary(self, pluginList, devicePath, simulatorPath, tagretPath):
        """
        合并插件的二进制
        :param pluginList: 插件列表
        :param devicePath: 真机架构二进制路径
        :param simulatorPath: 模拟器架构二进制路径
        :param tagretPath: 合并后的二进制路径
        """
        for plugin in pluginList:
            if plugin.startswith("."):
                continue
            pluginDevicePath = devicePath + "/lib" + plugin + ".a"
            pluginSimulatorPath = simulatorPath + "/lib" + plugin + ".a"
            pluginTargetPath = os.path.join(tagretPath, plugin)
            if not self.fileHandle.file_exist(pluginDevicePath):
                self.result.returnError(pluginDevicePath + "找不到真机的二进制")
            if not self.fileHandle.file_exist(pluginSimulatorPath):
                self.result.returnError(plugin + "找不到模拟器的二进制")
            returnCode, content = self.shell.excommand_until_done("lipo -create " + pluginDevicePath + " " + pluginSimulatorPath + " -output " + pluginTargetPath)
            if "error" in content:
                self.result.returnError("合并" + plugin + "失败:" + content)

    def createFramework(self, framework_file_path, plugin, binary_path):
        """
        为插件创建framework
        :param framework_file_path: framework的路径
        :param plugin: 插件名字
        :param binary_path: 插件二进制路径
        """
        # framework_file_path = os.path.join(framework_path, podName + ".framework")
        self.formatter.format_print(framework_file_path)
        if not self.fileHandle.file_exist(framework_file_path):
            self.formatter.format_print(framework_file_path)
            self.shell.excommand_until_done("mkdir -p " + framework_file_path)
        self.shell.excommand_until_done("pwd ")

        self.shell.excommand_until_done("cp -R -p " + binary_path + " " + framework_file_path + "/" + plugin)
        # 创建headers
        dest_headers_path = os.path.join(framework_file_path, "Headers")
        if not self.fileHandle.file_exist(dest_headers_path):
            self.shell.excommand_until_done("mkdir -p " + dest_headers_path)
        headers_source_root = self.arguments.projectPath +"/ios/pluginSource/%s/Classes/" % (plugin)
        file_list = self.fileHandle.myGlob(headers_source_root, True)
        for file in file_list:
            if file.endswith(".h"):
                self.shell.excommand_until_done("cp -R -p " + file + " " + dest_headers_path)

    def processDependency(self, contentDict):
        podName = self.arguments.podName
        dependencyDict = {}  # 支持subspec的时候，需要处理key
        if not contentDict:
            self.result.returnError("打包失败，请查看打包返回结果")
        for key, content in contentDict.items():
            dependency = {}
            contentList = content.split("\n")
            # -> Installing LJGravityImageView (0.1.6)
            for line in contentList:
                if not str(line).startswith("-> Installing "):
                    continue
                contentAndVersionList = line.split(" Installing ")[-1].split(" ")
                if len(contentAndVersionList) != 2:
                    self.result.returnError(line + "\n解析库和版本失败。")
                name = contentAndVersionList[0]
                if podName in name:
                    continue
                version = contentAndVersionList[-1].strip(")").strip("(")
                dependency[name] = version
            if dependency:
                dependencyDict[key] = dependency
        return dependencyDict

    def startPackage(self, subSpecName, debug):
        # 用到的参数
        projectPath = self.arguments.projectPath
        podName = self.arguments.podName
        if subSpecName:
            subspecs = self.podspecTool.subspecList()
        CAFPath = self.arguments.CAFPath
        isFrameWork = self.arguments.framework
        moduleSources = self.arguments.moduleSources
        version = self.arguments.version
        podspecName = podName + subSpecName + ".podspec"
        haveSourceFile = self.podspecTool.canPackage(podName, subSpecName)
        if not haveSourceFile:
            self.formatter.format_warning("没有源码, 不能打这个包 ")
            return "没有源码，打包失败"
        # 打包之前需要先更新repo源。
        moduleSourceList = self.arguments.moduleSourceList
        self.cocoaPodsTool.updateRepoList(moduleSourceList)
        # 备份一份
        self.podspecTool.backUpPodspec()

        # 改变成需要的podspec
        self.podspecTool.changePodSpecForPackage(subSpecName)
        if subSpecName:
            # 创建测试podspec
            self.podspecTool.redirectSubSpec(podName + subSpecName + ".podspec")
            self.cocoaPodsTool.create_tmp_podspec(subSpecName)
        cmd = ""
        # 只有自己的组件是源码，其他的最好是静态库
        if subSpecName and subspecs:
            for subspec in subspecs:
                cmd += podName + "_" + subspec + "_SOURCE=1 "
        else:
            cmd += podName + "_SOURCE=1 "
        if not debug:
            cmd += "IS_RELEASE=1 "
        prefixString = ''
        suffixString = ''
        productDir = podName
        if CAFPath:
            if not self.fileHandle.file_exist(CAFPath):
                self.formatter.format_warning("存放二进制目录不存在，正在创建存放源码二进制的目录")
                self.shell.excommand_until_done("mkdir -p " + CAFPath)
            productDir = os.path.join(CAFPath, podName)
        if isFrameWork:
            cmd += "pod package --verbose " + podspecName + " --force --no-mangle --verbose --exclude-deps --spec-sources="
            suffixString = ".framework"
            productDir = os.path.join(productDir, "Framework")
            cmd += moduleSources + " "
        destFileName = prefixString + podName + suffixString
        if len(subSpecName) > 0:
            productDir += "/" + subSpecName
            destFileName = prefixString + podName + subSpecName + suffixString
            cmd += "--subspecs=" + subSpecName
        productPath = ""
        if debug:
            productDir += "/Debug"
            productPath = productDir + "/" + destFileName
            cmd += " --configuration=Debug "
        else:
            productDir += "/Release"
            productPath = productDir + "/" + destFileName
        cmd += " --archs="+self.arguments.archs + " --archs_sim=" + self.arguments.archs_sim + " "
        self.formatter.format_info(cmd)
        returnCode, content = self.shell.excommand_until_done(cmd)
        if returnCode > 0 or "** BUILD FAILED **" in content or "Build command failed:" in content:
            self.result.returnError("打包失败，请搜索尖括号内 <error: > 或者 <error generated> 或者 <errors generated> 查看具体信息 ")
        self.formatter.format_print(version)
        buildPath = podName + subSpecName + "-" + version
        binaryFilePath = buildPath + "/ios/" + prefixString + podName + subSpecName + suffixString

        if not self.fileHandle.file_exist(productDir):
            self.shell.excommand_until_done("mkdir -p " + productDir)
        else:
            self.shell.excommand_until_done("rm -r -f " + productDir)
            self.shell.excommand_until_done("mkdir -p " + productDir)

        self.formatter.format_print(binaryFilePath)
        if not self.fileHandle.file_exist(binaryFilePath):
            return content
        if haveSourceFile:
            self.formatter.format_print("即将把" + destFileName + "放入发布目录：" + productDir)
            self.shell.excommand_until_done("cp -R -p \"" + binaryFilePath + "\" \"" + productPath + "\"")
            # 创建头文件软链
            self.formatter.format_print("即将把为静态库的创建软链：")
            os.chdir(productDir)
            cmd = "ln -s " + destFileName + "/**/*.h ./"
            self.shell.excommand_until_done(cmd)
            os.chdir(projectPath)

        self.formatter.format_print("即将删除临时文件")
        self.shell.excommand_until_done("rm -r -f " + buildPath)
        self.shell.excommand_until_done("rm -r -f " + podspecName)
        # 恢复最初的 podspec供下次使用
        self.podspecTool.restorePodspec()
        self.formatter.format_info("成功生成静态库")
        return content


#!/usr/bin/env python
# -*- coding=utf-8 -*-

import sys
import os
import getopt
import json
from commonFuncs import *
from constant import *
from formatter import Formatter
from result import Result
from fileHandle import FileHandle
from tips import Tips
from shellCommand import Shell
import re

class Arguments(object):
    __instance = None

    def __init__(self):
        self.formatter = Formatter.instance()
        self.result = Result.instance()
        self.fileHandle = FileHandle.instance()
        self.tips = Tips.instance()
        self.shell = Shell.instance()
        self._initArguments()
        self._arguments()
        self._checkArgues()

    @classmethod
    def instance(cls):
        if cls.__instance is None:
            cls.__instance = Arguments()
        return cls.__instance

    def _initArguments(self):

        # 调试信息
        # 调试信息要不要打开
        self.debug = False
        # 白名单信息，
        self.whiteListPath = ""

        # 项目信息
        # 用户名
        self.user = ""
        # 工程目录
        self.projectPath = ""
        # pod 名字
        self.podName = ""
        # 分支
        self.branch = ""
        # 将要打的版本
        self.version = ""
        # 将要打的commitID
        self.commitId = ""
        # pod库是否按subspec分别进行打包
        self.subPackage = False

        # 打包信息
        # 组件源有哪些，逗号隔开。若果有不同的APP，可以用组件池区分开。基础组件，A端，B端或者C端等等。
        self.moduleSources = ""
        # 依赖的json串，
        self.dependencyJSON = ""
        # 打包的时候是否用SSH 下载，有时候提交太大，需要用SSH
        self.useSSH = False
        # 打包前是否检查
        self.check = False
        # 要不要打framework静态库
        self.framework = False
        # 要不要打debug包
        self.debugPackage = False
        # 要不要打 Release包
        self.releasePackage = False
        # 真机架构
        self.archs = ""
        # 模拟器架构
        self.archs_sim = ""



        # 打包输出结果的文件路径，会输出JSON格式的输出
        self.resultPath = ""
        # 打包后源码和静态库存放的路径，如果不提供，则在工程目录下输出二进制结果
        self.CAFPath = ""

        # 是否自动吧提交信息push到远程
        self.autoPush = False

        # 发布组件
        # 是否自动发布repo
        self.autoPushRepo = False
        # 如果自动发布repo，要发布到哪里。多个用逗号隔开
        self.repoSources = ""
        # 是否提供大文件存储的URL，如果提供，则会在发布repo的时候改变podspec 里s.source 里的URL
        self.LFSURL = ""

        self.isflutter = False
        self.flaver = ""


    def _arguments(self):
        """
        获得参数
        :param type: 哪个命令的参数
        :return:
        """
        if type == "":
            return
        try:
            opts, args = getopt.getopt(sys.argv[2:], "",
                                       ["help", "user=", "projectPath=", "subPackage=", "podName=", "branch=",
                                        "version=", "commitId=", "debug=", "dependencyJSON=", "framework", "archive",
                                        "check", "debugPackage", "releasePackage", "autoPush", "autoPushRepo",
                                        "repoSources=", "resultPath=", "whiteListPath=", "subspecs=", "CAFPath=",
                                        "LFSURL=","moduleSources=", "updateMasterSource", "useSSH", "archs=", "archs_sim=",
                                        "isFlutter", "flaver="])
            # sys.argv[1:] 过滤掉第一个参数(它是脚本名称，不是参数的一部分)
            pass
        except getopt.GetoptError:
            self.formatter.format_error("argv error,please input")
            self.result.returnError()
        if not args and not opts:
            self.tips.showTips()
            self.result.returnError()
        # 使用一个循环，每次从opts中取出一个两元组，赋给两个变量。cmd保存选项参数，arg为附加参数。接着对取出的选项参数进行处理。
        for cmd, arg in opts:
            self.formatter.format_print(cmd + "  " + arg)
            if cmd == "--debug":
                self.debug = boolValue(arg)
            elif cmd == "--whiteListPath":
                self.whiteListPath = str(arg)
            elif cmd in ("--user"):
                self.user = str(arg)
            elif cmd in ("--projectPath"):
                self.projectPath = str(arg)
                self.projectName = self.projectPath.rstrip("/").split('/')[-1]
                self.projectParentDict = self.projectPath.replace(self.projectName, "")
            elif cmd == "--podName":
                self.podName = str(arg)
            elif cmd == "--branch":
                self.branch = str(arg)
            elif cmd in ("--version"):
                self.version = str(arg)
            elif cmd == "--commitId":
                self.commitId = str(arg)
            elif cmd == "--moduleSources":
                self.moduleSources = str(arg)
                if self.moduleSources:
                    self.moduleSourceList = self.moduleSources.strip(" ").split(",")
                else:
                    self.moduleSources = ""
                    self.moduleSourceList = []
            elif cmd == "--useSSH":
                self.useSSH = boolValue(arg)
            elif cmd == "--dependencyJSON":
                self.dependencyJSON = str(arg)
                if self.dependencyJSON:
                    self.dependencyDict = self.getDependencyDict()
                else:
                    self.dependencyDict = {}
            elif cmd == "--check":
                self.chdependeeck = boolValue(arg)
            elif cmd == "--subspecs":
                self.subspecs = str(arg)
            elif cmd == "--framework":
                self.framework = boolValue(arg)
            elif cmd == "--archive":
                self.archive = boolValue(arg)
            elif cmd == "--debugPackage":
                self.debugPackage = boolValue(arg)
            elif cmd == "--releasePackage":
                self.releasePackage = boolValue(arg)
            elif cmd == "--resultPath":
                self.resultPath = str(arg)
            elif cmd == "--CAFPath":
                self.CAFPath = str(arg)
            elif cmd == "--subPackage":
                self.subPackage = boolValue(arg)
            elif cmd == "--autoPush":
                self.autoPush = boolValue(arg)
            elif cmd == "--autoPushRepo":
                self.autoPushRepo = boolValue(arg)
            elif cmd == "--repoSources":
                self.repoSources = str(arg)
                if self.repoSources:
                    self.repoSourceList = self.repoSources.strip(" ").split(",")
                else:
                    self.repoSourceList = []
            elif cmd == "--LFSURL":
                self.LFSURL = str(arg)
            elif cmd == "--archs":
                self.archs = str(arg)
            elif cmd == "--archs_sim":
                self.archs_sim = str(arg)
            elif cmd == "--archs_sim":
                self.archs_sim = str(arg)
            elif cmd == "--isFlutter":
                self.isflutter = boolValue(arg)
            elif cmd == "--flaver":
                self.flaver = str(arg)
            else:
                self.tips.showTips()
                self.result.returnError()

    def _checkArgues(self):
        if not self.podName and self.projectPath:
            self.podName = self.projectName
        if self.version and self.framework and (self.debugPackage or self.releasePackage):
            if len(self.version.split(".")) > 4 and len(self.version.split(".")) < 3:
                self.result.returnError(self.version + "  输入的版本号不符合规范")
            match = re.match("^([0-9]\d|[0-9])(\.([0-9]\d|\d|[a-z])*){2,3}(-SNAPSHOT)?$", self.version, re.S)
            if not match:
                self.result.returnError(self.version + "  输入的版本号不符合规范")
            if self.projectPath:
                if os.path.isdir(self.projectPath):
                    os.chdir(self.projectPath)
                else:
                    self.result.returnError("找不到这个目录"+self.projectPath)
            returnCode, content = self.shell.excommand_until_done("git tag")
            if returnCode > 0:
                self.result.returnError("获得tag失败")
            if content:
                tagList = content.split("\n")
                tagString = self.podName + "-" + str(self.version)
                for tag in tagList:
                    if tagString == tag:
                        self.result.returnError("发现有相同的tag号, 请检查:" + tag)
            if len(self.archs) == 0 or len(self.archs_sim) == 0:
                self.result.returnError("archs 或者archs_sim 必须要都有架构")


    def getDependencyDict(self):
        if self.fileHandle.file_exist(self.dependencyJSON):
            # 文件形式
            self.dependencyJSON = self.fileHandle.readFile(self.dependencyJSON)
        dependencyDict = json.loads(self.dependencyJSON)
        return byteify(dependencyDict)

    def argumentDict(self):
        return {
            "user": self.user,"projectPath": self.projectPath,"podName": self.podName,
            "branch":self.branch,"version": self.version,"commitId":self.commitId,"moduleSources":self.moduleSources,
            "dependencyJSON": self.dependencyJSON,"useSSH":self.useSSH,"check":self.check,
            "framework":self.framework, "debugPackage":self.debugPackage,
            "releasePackage":self.releasePackage, "resultPath":self.resultPath, "CAFPath": self.CAFPath,
            "autoPush": self.autoPush, "autoPushRepo": self.autoPushRepo, "repoSources":self.repoSources,
            "LFSURL":self.LFSURL,
        }


